/*
 * AlgorithmForm.java
 *
 * Created on Utorok, 2008, aprl 1, 22:28
 */

package form;

import algorithm.Algorithm;
import algorithm.SourceCode;
import form.dialog.CodeSettingsDialog;
import form.dialog.StandardVisualizationDataDialog;
import form.dialog.TextDialog;
import graphics.StandardVisualizationData;
import java.awt.FontMetrics;
import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;
import javax.swing.DefaultListModel;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.table.DefaultTableModel;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.text.TabSet;
import javax.swing.text.TabStop;
import network.Network;
import other.Functions;
import other.Global;

/**
 *
 * @author  Dur
 */
public class AlgorithmForm extends javax.swing.JFrame {
  protected JFrame caller;
  protected Algorithm algorithm;
  protected Style cleanStyle;
  protected HashMap<String, Style> wordStyles;
  protected HashMap<String, String> ignoreHighlight;
  protected int jTabbedPane1Index=0;
  
  /** Creates new form AlgorithmForm */
  private void InitForm()
  {
    setExtendedState(this.MAXIMIZED_BOTH);
    if (algorithm!=null)
    {
      //get network list
      DefaultListModel model=new DefaultListModel();
      String[] networkList=algorithm.GetNetworkList();
      for (int i=0;i<networkList.length;i++)
      {
        model.addElement(networkList[i]);
      }
      jList1.setModel(model);
    }
    jList1ValueChanged(null);
    RefreshTable1();
    setTabs(jTextPane1, 2);
    
    wordStyles=GetWordStyles();
    StyledDocument doc = (StyledDocument)jTextPane1.getDocument();
    cleanStyle=doc.addStyle("clean", null);
    StyleConstants.setBold(cleanStyle, false);
    
    ignoreHighlight=new HashMap<String, String>();
    ignoreHighlight.put("\"", "\"");
    ignoreHighlight.put("'", "'");
    ignoreHighlight.put("//", "\n");
    ignoreHighlight.put("/*", "*/");
    
    jTextPane1.setText(algorithm.GetSourceCode().GetCode());
    HighlightKeyWords();
  }
  
  public AlgorithmForm(JFrame caller,Algorithm algorithm) {
    this.caller=caller;
    this.algorithm=algorithm;
    initComponents();
    jTable1.getSelectionModel().addListSelectionListener(new ListSelectionListener() 
    {
      public void valueChanged(ListSelectionEvent e) {
        if (jTable1.getSelectedRow()==-1)
        {
          jButton7.setEnabled(false);
          jButton8.setEnabled(false);
          jButton9.setEnabled(false);
        }
        else
        {
          jButton7.setEnabled(true);
          jButton8.setEnabled(true);
          jButton9.setEnabled(true);
        }
      }
    });
    InitForm();
  }
  
  public String GetPreviousRowSpaces()
  {
    try
    {
      int actualLine=Functions.CountOccurances(jTextPane1.getText().substring(0, jTextPane1.getCaretPosition()-1), "\r");
      String[] lines=Functions.Explode("\r", jTextPane1.getText());
      int pos=0;
      while ((pos<lines[actualLine].length()) && ((lines[actualLine].substring(pos, pos+1).equals(" ")) || (lines[actualLine].substring(pos, pos+1).equals("\t"))))
      {
        pos++;
      }
      return lines[actualLine].substring(0, pos);
    }
    catch (Exception e)
    {
      Global.errorLog.Write("Error: form->AlgorithmForm->GetPreviousRowSpaces", e);
    }
    return "";
  }
  
  public HashMap<String, Style> GetWordStyles()
  {
    HashMap<String, Style> res=new HashMap<String, Style>();
    StyledDocument doc=(StyledDocument)jTextPane1.getDocument();
    
    Style bold=doc.addStyle("bold", null);
    StyleConstants.setBold(bold, true);
    
    String[] boldWords=new String[]{"if","else","for","while","do","return","import",
      "class","public","protected","private","new","null","try","catch","finally","throw",
      "static","final","extends","implements","void","instanceof","abstract","synchronized",
      "SendMessageByLink","SendMessageToNode","GetLinksCount","LinkExists","Assert",
      "GetNodesCount","NodeExists","GetNodesIds","SetProperty","GetProperty","GetId",
      "Run","ReceiveMessage","SetMessageReceivedListener","StopProcess","switch","case",
      "break"};
    for (int i=0;i<boldWords.length;i++)
    {
      res.put(boldWords[i], bold);
    }
    
    return res;
  }
  
  public void RefreshTable1()
  {
    DefaultTableModel model=(DefaultTableModel)jTable1.getModel();
    model.setNumRows(0);
    for (int i=0;i<algorithm.visualizationData.length;i++)
    {
      Object[] o=new Object[2];
      o[0]=algorithm.visualizationData[i].properties.get("name");
      o[1]="No";
      if (i==algorithm.activatedVisualizationDataIndex)
        o[1]="Yes";
      model.addRow(o);
    }
  }
  
  public void HighlightKeyWords()
  {
    try
    {
      //highlight key words
      StyledDocument doc = (StyledDocument)jTextPane1.getDocument();
      String s=doc.getText(0, doc.getLength());
      doc.setCharacterAttributes(0, doc.getLength(), cleanStyle, true);
      Iterator<Entry<String, Style>> it=wordStyles.entrySet().iterator();
      while (it.hasNext())
      {
        Entry<String, Style> e=it.next();
        Iterator<Integer> it2=Functions.GetSubstringPositions(s,e.getKey()).iterator();
        while (it2.hasNext())
        {
          doc.setCharacterAttributes(it2.next(), e.getKey().length(), e.getValue(), true);
        }
      }
      
      //erase some highlights
      Iterator<Entry<String, String>> it2=ignoreHighlight.entrySet().iterator();
      while (it2.hasNext())
      {
        Entry<String, String> e=it2.next();
        boolean repeat=true;
        int pos1=0;
        int pos2=-e.getValue().length();
        while (repeat)
        {
          pos1=s.indexOf(e.getKey(),pos2+e.getValue().length());
          repeat=false;
          if (pos1>-1)
          {
            pos2=s.indexOf(e.getValue(),pos1+e.getKey().length());
            if (pos2>-1)
            {
              repeat=true;
              doc.setCharacterAttributes(pos1, pos2-pos1+e.getValue().length(), cleanStyle, true);
            }
          }
        }
      }
    }
    catch (Exception e)
    {
      Global.errorLog.Write("Error: form->AlgorithmForm->HighlightKeyWords", e);
    }
  }
  
  public void ShowPosition()
  {
    try
    {
      int x=1,y=0;
      String s=jTextPane1.getText();
      int caret=jTextPane1.getCaretPosition();
      int pos=0;
      int pos1=s.indexOf("\r",pos);
      while ((pos1>-1) && (pos1<caret))
      {
        x++;
        pos=pos1+1;
        pos1=s.indexOf("\r",pos);
      }
      y=caret-pos+1;
      jLabel1.setText("Position: "+x+","+y);
    }
    catch (Exception e)
    {
      Global.errorLog.Write("Error: form->AlgorithmForm->ShowPosition", e);
    }
  }
  
  public void setTabs(JTextPane textPane, int charactersPerTab)
	{
		FontMetrics fm = textPane.getFontMetrics( textPane.getFont() );
		int charWidth = fm.charWidth( 'w' );
		int tabWidth = charWidth * charactersPerTab;
 
		TabStop[] tabs = new TabStop[10];
 
		for (int j = 0; j < tabs.length; j++)
		{
			int tab = j + 1;
			tabs[j] = new TabStop( tab * tabWidth );
		}
 
		TabSet tabSet = new TabSet(tabs);
		SimpleAttributeSet attributes = new SimpleAttributeSet();
		StyleConstants.setTabSet(attributes, tabSet);
		int length = textPane.getDocument().getLength();
		textPane.getStyledDocument().setParagraphAttributes(0, length, attributes, true);
	}
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
  private void initComponents() {

    jTabbedPane1 = new javax.swing.JTabbedPane();
    jPanel1 = new javax.swing.JPanel();
    jPanel3 = new javax.swing.JPanel();
    jButton1 = new javax.swing.JButton();
    jButton5 = new javax.swing.JButton();
    jButton3 = new javax.swing.JButton();
    jButton2 = new javax.swing.JButton();
    jButton4 = new javax.swing.JButton();
    jScrollPane1 = new javax.swing.JScrollPane();
    jList1 = new javax.swing.JList();
    jPanel2 = new javax.swing.JPanel();
    jPanel6 = new javax.swing.JPanel();
    jButton13 = new javax.swing.JButton();
    jButton10 = new javax.swing.JButton();
    jButton11 = new javax.swing.JButton();
    jButton12 = new javax.swing.JButton();
    jPanel7 = new javax.swing.JPanel();
    jScrollPane2 = new javax.swing.JScrollPane();
    jTextPane1 = new javax.swing.JTextPane();
    jPanel8 = new javax.swing.JPanel();
    jLabel1 = new javax.swing.JLabel();
    jPanel4 = new javax.swing.JPanel();
    jScrollPane3 = new javax.swing.JScrollPane();
    jTable1 = new javax.swing.JTable();
    jPanel5 = new javax.swing.JPanel();
    jButton6 = new javax.swing.JButton();
    jButton7 = new javax.swing.JButton();
    jButton8 = new javax.swing.JButton();
    jButton9 = new javax.swing.JButton();

    setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
    setTitle("VDA");
    setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
    addWindowListener(new java.awt.event.WindowAdapter() {
      public void windowClosing(java.awt.event.WindowEvent evt) {
        formWindowClosing(evt);
      }
    });

    jTabbedPane1.addChangeListener(new javax.swing.event.ChangeListener() {
      public void stateChanged(javax.swing.event.ChangeEvent evt) {
        jTabbedPane1StateChanged(evt);
      }
    });

    jPanel1.setLayout(new java.awt.BorderLayout());

    jPanel3.setLayout(new java.awt.GridLayout(1, 0, 5, 0));

    jButton1.setText("New network");
    jButton1.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton1ActionPerformed(evt);
      }
    });
    jPanel3.add(jButton1);

    jButton5.setText("Rename network");
    jButton5.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton5ActionPerformed(evt);
      }
    });
    jPanel3.add(jButton5);

    jButton3.setText("Delete network");
    jButton3.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton3ActionPerformed(evt);
      }
    });
    jPanel3.add(jButton3);

    jButton2.setText("Network editor");
    jButton2.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton2ActionPerformed(evt);
      }
    });
    jPanel3.add(jButton2);

    jButton4.setText("Run visualization");
    jButton4.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton4ActionPerformed(evt);
      }
    });
    jPanel3.add(jButton4);

    jPanel1.add(jPanel3, java.awt.BorderLayout.SOUTH);

    jList1.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
    jList1.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
      public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
        jList1ValueChanged(evt);
      }
    });
    jScrollPane1.setViewportView(jList1);

    jPanel1.add(jScrollPane1, java.awt.BorderLayout.CENTER);

    jTabbedPane1.addTab("Network list", jPanel1);

    jPanel2.setLayout(new java.awt.BorderLayout());

    jPanel6.setLayout(new java.awt.GridLayout(1, 0, 5, 0));

    jButton13.setText("Save");
    jButton13.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton13ActionPerformed(evt);
      }
    });
    jPanel6.add(jButton13);

    jButton10.setText("Load from file");
    jButton10.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton10ActionPerformed(evt);
      }
    });
    jPanel6.add(jButton10);

    jButton11.setText("Empty algorithm");
    jButton11.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton11ActionPerformed(evt);
      }
    });
    jPanel6.add(jButton11);

    jButton12.setText("Settings");
    jButton12.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton12ActionPerformed(evt);
      }
    });
    jPanel6.add(jButton12);

    jPanel2.add(jPanel6, java.awt.BorderLayout.PAGE_END);

    jPanel7.setLayout(new java.awt.BorderLayout());

    jTextPane1.setFont(new java.awt.Font("Courier New", 0, 11));
    jTextPane1.addKeyListener(new java.awt.event.KeyAdapter() {
      public void keyReleased(java.awt.event.KeyEvent evt) {
        jTextPane1KeyReleased(evt);
      }
    });
    jTextPane1.addCaretListener(new javax.swing.event.CaretListener() {
      public void caretUpdate(javax.swing.event.CaretEvent evt) {
        jTextPane1CaretUpdate(evt);
      }
    });
    jScrollPane2.setViewportView(jTextPane1);

    jPanel7.add(jScrollPane2, java.awt.BorderLayout.CENTER);

    jPanel8.setLayout(new javax.swing.BoxLayout(jPanel8, javax.swing.BoxLayout.LINE_AXIS));

    jLabel1.setText("Position: ");
    jPanel8.add(jLabel1);

    jPanel7.add(jPanel8, java.awt.BorderLayout.PAGE_END);

    jPanel2.add(jPanel7, java.awt.BorderLayout.CENTER);

    jTabbedPane1.addTab("Source code", jPanel2);

    jPanel4.setLayout(new java.awt.BorderLayout());

    jTable1.setModel(new javax.swing.table.DefaultTableModel(
      new Object [][] {

      },
      new String [] {
        "Name", "Activated"
      }
    ) {
      boolean[] canEdit = new boolean [] {
        false, false
      };

      public boolean isCellEditable(int rowIndex, int columnIndex) {
        return canEdit [columnIndex];
      }
    });
    jTable1.addMouseListener(new java.awt.event.MouseAdapter() {
      public void mouseClicked(java.awt.event.MouseEvent evt) {
        jTable1MouseClicked(evt);
      }
    });
    jScrollPane3.setViewportView(jTable1);

    jPanel4.add(jScrollPane3, java.awt.BorderLayout.CENTER);

    jPanel5.setLayout(new java.awt.GridLayout(1, 0, 5, 0));

    jButton6.setText("Add");
    jButton6.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton6ActionPerformed(evt);
      }
    });
    jPanel5.add(jButton6);

    jButton7.setText("Edit");
    jButton7.setEnabled(false);
    jButton7.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton7ActionPerformed(evt);
      }
    });
    jPanel5.add(jButton7);

    jButton8.setText("Delete");
    jButton8.setEnabled(false);
    jButton8.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton8ActionPerformed(evt);
      }
    });
    jPanel5.add(jButton8);

    jButton9.setText("Activate");
    jButton9.setEnabled(false);
    jButton9.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton9ActionPerformed(evt);
      }
    });
    jPanel5.add(jButton9);

    jPanel4.add(jPanel5, java.awt.BorderLayout.PAGE_END);

    jTabbedPane1.addTab("Visualization", jPanel4);

    getContentPane().add(jTabbedPane1, java.awt.BorderLayout.CENTER);

    java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
    setBounds((screenSize.width-729)/2, (screenSize.height-441)/2, 729, 441);
  }// </editor-fold>//GEN-END:initComponents

  private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
    if ((algorithm.activatedVisualizationDataIndex>=0) && (algorithm.activatedVisualizationDataIndex<algorithm.visualizationData.length))
    {
      int index=jList1.getSelectedIndex();
      algorithm.networks[index].OpenEditorForm(this);
      setVisible(false);
    }
    else
    {
      JOptionPane.showMessageDialog(this, "No visualization is activated.", "Information", JOptionPane.INFORMATION_MESSAGE);
    }
  }//GEN-LAST:event_jButton2ActionPerformed

  private void jList1ValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_jList1ValueChanged
    if (jList1.getSelectedIndex()==-1)
    {
      jButton2.setEnabled(false);
      jButton3.setEnabled(false);
      jButton4.setEnabled(false);
      jButton5.setEnabled(false);
    }
    else
    {
      jButton2.setEnabled(true);
      jButton3.setEnabled(true);
      jButton4.setEnabled(true);
      jButton5.setEnabled(true);
    }
  }//GEN-LAST:event_jList1ValueChanged

  private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
    if (JOptionPane.showConfirmDialog(this,"Do you really want to delete selected network?","Delete",JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION)
    {
      int index=jList1.getSelectedIndex();
      DefaultListModel model=(DefaultListModel) jList1.getModel();
      model.removeElementAt(index);
      algorithm.DeleteNetwork(index);
      if (index>(algorithm.networks.length-1)) index--;
      jList1.setSelectedIndex(index);
    }
  }//GEN-LAST:event_jButton3ActionPerformed

  private void jButton5ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton5ActionPerformed
    int index=jList1.getSelectedIndex();
    Network net=algorithm.networks[index];
    net.GetName();
    String name=JOptionPane.showInputDialog("New network name",net.GetName());
    if (!name.equals(""))
    {
      net.SetName(name);
      DefaultListModel model=(DefaultListModel) jList1.getModel();
      model.removeElementAt(index);
      model.add(index,name);
      jList1.setModel(model);
      jList1.setSelectedIndex(index);
    }
  }//GEN-LAST:event_jButton5ActionPerformed

  private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
    String name=JOptionPane.showInputDialog("Network name","new_network");
    if (!name.equals(""))
    {
      Network net=algorithm.NewNetwork();
      net.SetName(name);
      DefaultListModel model=(DefaultListModel) jList1.getModel();
      model.addElement(name);
      jList1.setModel(model);
      jList1.setSelectedIndex(algorithm.networks.length-1);
    }
  }//GEN-LAST:event_jButton1ActionPerformed

  private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
    algorithm.GetSourceCode().SetCode(jTextPane1.getText());
    Global.algorithmsCollection.Save();
    if (caller!=null)
    {
      caller.setVisible(true);
    }
    dispose();
  }//GEN-LAST:event_formWindowClosing

private void jButton4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton4ActionPerformed
  if ((algorithm.activatedVisualizationDataIndex>=0) && (algorithm.activatedVisualizationDataIndex<algorithm.visualizationData.length))
  {
    Algorithm algCopy=algorithm.GetCopy();
    if (algCopy.GetSourceCode().Compile())
    {
      int index=jList1.getSelectedIndex();
      algCopy.networks[index].OpenVisualizationForm(this);
      setVisible(false);
    }
    else
    {
      TextDialog dial=new TextDialog(algCopy.GetSourceCode().GetCompilationError(), this, true);
      dial.setTitle("Compilation error");
      dial.setVisible(true);
    }
  }
  else
  {
    JOptionPane.showMessageDialog(this, "No visualization is activated.", "Information", JOptionPane.INFORMATION_MESSAGE);
  }
}//GEN-LAST:event_jButton4ActionPerformed

private void jButton9ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton9ActionPerformed
  algorithm.activatedVisualizationDataIndex=jTable1.getSelectedRow();
  RefreshTable1();
}//GEN-LAST:event_jButton9ActionPerformed

private void jButton6ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton6ActionPerformed
  StandardVisualizationDataDialog dial=new StandardVisualizationDataDialog(this, true);
  dial.setVisible(true);
  if (dial.IsOKPressed())
  {
    StandardVisualizationData[] newArray=new StandardVisualizationData[algorithm.visualizationData.length+1];
    System.arraycopy(algorithm.visualizationData, 0, newArray, 0, algorithm.visualizationData.length);
    newArray[newArray.length-1]=dial.GetVisualizationData();
    algorithm.visualizationData=newArray;
  }
  RefreshTable1();
}//GEN-LAST:event_jButton6ActionPerformed

private void jButton7ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton7ActionPerformed
  StandardVisualizationDataDialog dial=new StandardVisualizationDataDialog(algorithm.visualizationData[jTable1.getSelectedRow()], this, true);
  dial.setVisible(true);
  if (dial.IsOKPressed())
  {
    algorithm.visualizationData[jTable1.getSelectedRow()]=dial.GetVisualizationData();
    RefreshTable1();
  }
}//GEN-LAST:event_jButton7ActionPerformed

private void jButton8ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton8ActionPerformed
  if (JOptionPane.showConfirmDialog(this, "Are you sure?","Delete",JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION)
  {
    if (algorithm.activatedVisualizationDataIndex>=jTable1.getSelectedRow())
      algorithm.activatedVisualizationDataIndex--;
    StandardVisualizationData[] newArray=new StandardVisualizationData[algorithm.visualizationData.length-1];
    int pos=0;
    for (int i=0;i<algorithm.visualizationData.length;i++)
    {
      if (i!=jTable1.getSelectedRow())
      {
        newArray[pos]=algorithm.visualizationData[i];
        pos++;
      }
    }
    algorithm.visualizationData=newArray;
    if (algorithm.visualizationData.length<1)
      algorithm.activatedVisualizationDataIndex=0;
    RefreshTable1();
  }
}//GEN-LAST:event_jButton8ActionPerformed

private void jTable1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jTable1MouseClicked
  if ((jTable1.getSelectedRow()>=0) && (evt!=null) && (evt.getClickCount()>1))
  {
    jButton7ActionPerformed(null);
  }
}//GEN-LAST:event_jTable1MouseClicked

private void jTextPane1KeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextPane1KeyReleased
  if ((evt!=null) && (evt.getKeyChar()=='\n'))
  {
    int pos=jTextPane1.getCaretPosition();
    jTextPane1.setText(jTextPane1.getText().replaceAll("\n", "\r"));
    jTextPane1.setCaretPosition(pos);
    String spaces=GetPreviousRowSpaces();
    String text=jTextPane1.getText();
    String left="";
    if (pos>0)
      left=text.substring(0,pos);
    String right="";
    if (pos<text.length())
      right=text.substring(pos);
    jTextPane1.setText(left+spaces+right);
    jTextPane1.setCaretPosition(left.length()+spaces.length());
  }
  HighlightKeyWords();
  jButton13.setEnabled(true);
}//GEN-LAST:event_jTextPane1KeyReleased

private void jButton11ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton11ActionPerformed
  if (JOptionPane.showConfirmDialog(this, "Do you really want to replace existing algorithm with empty?", "Clear", JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION)
  {
    jTextPane1.setText(SourceCode.GetEmptyAlgorithm());
    HighlightKeyWords();
  }
}//GEN-LAST:event_jButton11ActionPerformed

private void jButton10ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton10ActionPerformed
  try
  {
    JFileChooser fileChooser = new JFileChooser();
    fileChooser.setFileFilter(new FileNameExtensionFilter("JAVA file", "java"));
    File file;    
    fileChooser.showDialog(this, "Load from file");
    file=fileChooser.getSelectedFile();
    if (file!=null)
    {
      String s=Functions.ReadFile(file);
      if (s!=null)
      {
        jTextPane1.setText(s);
        HighlightKeyWords();
      }
      else
      {
        JOptionPane.showMessageDialog(this, "Error occured during reading file.", "Error", JOptionPane.INFORMATION_MESSAGE);
      }
    }
  }
  catch (Exception e)
  {
    Global.errorLog.Write("Error: form->AlgorithmForm->jButton10ActionPerformed", e);
  }
}//GEN-LAST:event_jButton10ActionPerformed

private void jTabbedPane1StateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jTabbedPane1StateChanged
  if (jTabbedPane1Index==1)
  {
    algorithm.GetSourceCode().SetCode(jTextPane1.getText());
  }
  jTabbedPane1Index=jTabbedPane1.getSelectedIndex();
  jButton13.setEnabled(true);
}//GEN-LAST:event_jTabbedPane1StateChanged

private void jButton12ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton12ActionPerformed
  try
  {
    CodeSettingsDialog dial=new CodeSettingsDialog(this, true);
    dial.setVisible(true);
  }
  catch (Exception e)
  {
    Global.errorLog.Write("Error: form->AlgorithmForm->jButton12ActionPerformed", e);
  }
}//GEN-LAST:event_jButton12ActionPerformed

private void jButton13ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton13ActionPerformed
  algorithm.GetSourceCode().SetCode(jTextPane1.getText());
  Global.algorithmsCollection.Save();
  jButton13.setEnabled(false);
}//GEN-LAST:event_jButton13ActionPerformed

private void jTextPane1CaretUpdate(javax.swing.event.CaretEvent evt) {//GEN-FIRST:event_jTextPane1CaretUpdate
  ShowPosition();
}//GEN-LAST:event_jTextPane1CaretUpdate
  
  /**
   * @param args the command line arguments
   */
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  public javax.swing.JButton jButton1;
  public javax.swing.JButton jButton10;
  public javax.swing.JButton jButton11;
  public javax.swing.JButton jButton12;
  public javax.swing.JButton jButton13;
  public javax.swing.JButton jButton2;
  public javax.swing.JButton jButton3;
  public javax.swing.JButton jButton4;
  public javax.swing.JButton jButton5;
  public javax.swing.JButton jButton6;
  public javax.swing.JButton jButton7;
  public javax.swing.JButton jButton8;
  public javax.swing.JButton jButton9;
  public javax.swing.JLabel jLabel1;
  public javax.swing.JList jList1;
  public javax.swing.JPanel jPanel1;
  public javax.swing.JPanel jPanel2;
  public javax.swing.JPanel jPanel3;
  public javax.swing.JPanel jPanel4;
  public javax.swing.JPanel jPanel5;
  public javax.swing.JPanel jPanel6;
  public javax.swing.JPanel jPanel7;
  public javax.swing.JPanel jPanel8;
  public javax.swing.JScrollPane jScrollPane1;
  public javax.swing.JScrollPane jScrollPane2;
  public javax.swing.JScrollPane jScrollPane3;
  public javax.swing.JTabbedPane jTabbedPane1;
  public javax.swing.JTable jTable1;
  public javax.swing.JTextPane jTextPane1;
  // End of variables declaration//GEN-END:variables
  
}
