/*
 * NodeDialog.java
 *
 * Created on Nedea, 2008, mj 4, 23:36
 */

package form.dialog;

import java.util.Collections;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import javax.swing.JOptionPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;
import network.Node;
import other.AssociativeArray;
import other.Functions;
import other.Global;
import other.IDialog;

/**
 *
 * @author  Dur
 */
public class NodeDialog extends javax.swing.JDialog implements IDialog
{
  protected Node node;
  protected boolean isOKPressed=false,runningVisualization;
  protected AssociativeArray<Object, Object> properties;
  protected Map<Object, Object>nodeThreadProps;
  
  /**
   * Creates new form NodeDialog
   */
  
  private NodeDialog(java.awt.Frame parent, boolean modal) 
  {
    super(parent, modal);
    initComponents();
    jTable1.getSelectionModel().addListSelectionListener(new ListSelectionListener() 
    {
      public void valueChanged(ListSelectionEvent e) {
        if (jTable1.getSelectedRow()==-1)
        {
          jButton4.setEnabled(false);
          jButton5.setEnabled(false);
          jMenuItem2.setEnabled(false);
          jMenuItem3.setEnabled(false);
        }
        else
        {
          jButton4.setEnabled(true);
          jButton5.setEnabled(true);
          jMenuItem2.setEnabled(true);
          jMenuItem3.setEnabled(true);
        }
      }
    });
    getRootPane().setDefaultButton(jButton1);
  }
  
  public NodeDialog(Node node, java.awt.Frame parent, boolean modal, boolean runningVisualization) {
    this(parent, modal);
    this.node=node;
    this.runningVisualization=runningVisualization;
    this.properties=(AssociativeArray<Object, Object>)node.prop.clone();
    if (runningVisualization)
    {
      nodeThreadProps=new AssociativeArray<Object, Object>();
      synchronized (node.nodeThread.prop)
      {
        Iterator<Entry<Object, Object>> it=node.nodeThread.prop.entrySet().iterator();
        while (it.hasNext())
        {
          Entry<Object, Object> e=it.next();
          nodeThreadProps.put(e.getKey(), e.getValue());
        }
      }
      nodeThreadProps=Collections.synchronizedMap(nodeThreadProps);
      SetFormReadonly();
    }
    UpdateForm();
  }
  
  public void SetFormReadonly()
  {
    try
    {
      jFormattedTextField1.setEnabled(false);
    }
    catch (Exception e)
    {
      Global.errorLog.Write("form->dialog->NodeDialog->SetFormReadonly",e);
    }
  }
  
  public void UpdateForm()
  {
    try
    {
      if (node!=null)
      {
        if (!runningVisualization)
        {
          jFormattedTextField1.setValue(Functions.intVal(node.prop.get("id").toString()));
          DefaultTableModel model=(DefaultTableModel) jTable1.getModel();
          model.setNumRows(0);
          Iterator<Entry<Object, Object>> it=node.prop.entrySet().iterator();
          while (it.hasNext())
          {
            Entry<Object, Object> e=it.next();
            if (!e.getKey().equals("id"))
            {
              Object[] o=new Object[2];
              o[0]=e.getKey();
              o[1]=e.getValue();
              model.addRow(o);
            }
          }
        }
        else
        {
          synchronized (node.nodeThread.prop)
          {
            jFormattedTextField1.setValue(Functions.intVal(node.nodeThread.prop.get("id").toString()));
            DefaultTableModel model=(DefaultTableModel) jTable1.getModel();
            model.setNumRows(0);
            Iterator<Entry<Object, Object>> it=node.nodeThread.prop.entrySet().iterator();
            while (it.hasNext())
            {
              Entry<Object, Object> e=it.next();
              if (!e.getKey().equals("id"))
              {
                Object[] o=new Object[2];
                o[0]=e.getKey();
                o[1]=e.getValue();
                model.addRow(o);
              }
            }
          }
        }
      }
    }
    catch (Exception e)
    {
      Global.errorLog.Write("form->dialog->NodeDialog->UpdateForm",e);
    }
  }
  
  public void UpdateNode()
  {
    try
    {
      if (node!=null)
      {
        node.prop.put("id", jFormattedTextField1.getValue().toString());
      }
    }
    catch (Exception e)
    {
      Global.errorLog.Write("form->dialog->NodeDialog->UpdateNode",e);
    }
  }
  
  public boolean IsOKPressed()
  {
    return isOKPressed;
  }
  
  @Override
  public void setVisible(boolean b)
  {
    super.setVisible(b);
  }
  
  protected boolean IsIdInUse(int id)
  {
    boolean res=false;
    for (int i=0;i<node.network.nodes.length;i++)
    {
      if (node.network.nodes[i]!=node)
      {
        res|=(Functions.intVal(node.network.nodes[i].prop.get("id").toString())==id);
      }
    }
    return res;
  }
  
  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
  private void initComponents() {

    jPopupMenu1 = new javax.swing.JPopupMenu();
    jMenuItem1 = new javax.swing.JMenuItem();
    jMenuItem2 = new javax.swing.JMenuItem();
    jMenuItem3 = new javax.swing.JMenuItem();
    jPanel1 = new javax.swing.JPanel();
    jButton1 = new javax.swing.JButton();
    jButton2 = new javax.swing.JButton();
    jPanel8 = new javax.swing.JPanel();
    jPanel6 = new javax.swing.JPanel();
    jPanel2 = new javax.swing.JPanel();
    jPanel3 = new javax.swing.JPanel();
    jPanel4 = new javax.swing.JPanel();
    jLabel1 = new javax.swing.JLabel();
    jPanel5 = new javax.swing.JPanel();
    jFormattedTextField1 = new javax.swing.JFormattedTextField();
    jPanel12 = new javax.swing.JPanel();
    jPanel13 = new javax.swing.JPanel();
    jButton3 = new javax.swing.JButton();
    jButton4 = new javax.swing.JButton();
    jButton5 = new javax.swing.JButton();
    jScrollPane1 = new javax.swing.JScrollPane();
    jTable1 = new javax.swing.JTable();
    jPanel9 = new javax.swing.JPanel();
    jPanel7 = new javax.swing.JPanel();

    jMenuItem1.setText("New property");
    jMenuItem1.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jMenuItem1ActionPerformed(evt);
      }
    });
    jPopupMenu1.add(jMenuItem1);

    jMenuItem2.setText("Edit property");
    jMenuItem2.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jMenuItem2ActionPerformed(evt);
      }
    });
    jPopupMenu1.add(jMenuItem2);

    jMenuItem3.setText("Delete property");
    jMenuItem3.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jMenuItem3ActionPerformed(evt);
      }
    });
    jPopupMenu1.add(jMenuItem3);

    setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
    setTitle("Node properties");
    setResizable(false);
    addWindowListener(new java.awt.event.WindowAdapter() {
      public void windowClosing(java.awt.event.WindowEvent evt) {
        formWindowClosing(evt);
      }
    });

    jButton1.setText("OK");
    jButton1.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton1ActionPerformed(evt);
      }
    });
    jPanel1.add(jButton1);

    jButton2.setText("Cancel");
    jButton2.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton2ActionPerformed(evt);
      }
    });
    jPanel1.add(jButton2);

    getContentPane().add(jPanel1, java.awt.BorderLayout.PAGE_END);
    getContentPane().add(jPanel8, java.awt.BorderLayout.LINE_END);
    getContentPane().add(jPanel6, java.awt.BorderLayout.WEST);

    jPanel2.setLayout(new java.awt.BorderLayout());

    jPanel3.setLayout(new java.awt.BorderLayout());

    jPanel4.setLayout(new java.awt.GridLayout(0, 1, 0, 3));

    jLabel1.setText("Id: ");
    jPanel4.add(jLabel1);

    jPanel3.add(jPanel4, java.awt.BorderLayout.WEST);

    jPanel5.setLayout(new java.awt.GridLayout(0, 1, 0, 3));

    jFormattedTextField1.setFormatterFactory(new javax.swing.text.DefaultFormatterFactory(new javax.swing.text.NumberFormatter(new java.text.DecimalFormat("#0"))));
    jPanel5.add(jFormattedTextField1);

    jPanel3.add(jPanel5, java.awt.BorderLayout.CENTER);

    jPanel2.add(jPanel3, java.awt.BorderLayout.PAGE_START);

    jPanel12.setLayout(new java.awt.BorderLayout());

    jButton3.setText("New property");
    jButton3.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton3ActionPerformed(evt);
      }
    });
    jPanel13.add(jButton3);

    jButton4.setText("Edit property");
    jButton4.setEnabled(false);
    jButton4.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton4ActionPerformed(evt);
      }
    });
    jPanel13.add(jButton4);

    jButton5.setText("Delete property");
    jButton5.setEnabled(false);
    jButton5.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        jButton5ActionPerformed(evt);
      }
    });
    jPanel13.add(jButton5);

    jPanel12.add(jPanel13, java.awt.BorderLayout.PAGE_END);

    jTable1.setModel(new javax.swing.table.DefaultTableModel(
      new Object [][] {

      },
      new String [] {
        "Property", "Value"
      }
    ) {
      boolean[] canEdit = new boolean [] {
        false, false
      };

      public boolean isCellEditable(int rowIndex, int columnIndex) {
        return canEdit [columnIndex];
      }
    });
    jTable1.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
    jTable1.addMouseListener(new java.awt.event.MouseAdapter() {
      public void mouseClicked(java.awt.event.MouseEvent evt) {
        jTable1MouseClicked(evt);
      }
      public void mouseReleased(java.awt.event.MouseEvent evt) {
        jTable1MouseReleased(evt);
      }
    });
    jScrollPane1.setViewportView(jTable1);

    jPanel12.add(jScrollPane1, java.awt.BorderLayout.CENTER);
    jPanel12.add(jPanel9, java.awt.BorderLayout.PAGE_START);

    jPanel2.add(jPanel12, java.awt.BorderLayout.CENTER);

    getContentPane().add(jPanel2, java.awt.BorderLayout.CENTER);
    getContentPane().add(jPanel7, java.awt.BorderLayout.PAGE_START);

    java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
    setBounds((screenSize.width-400)/2, (screenSize.height-298)/2, 400, 298);
  }// </editor-fold>//GEN-END:initComponents

private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
  node.prop=properties;
  if (runningVisualization)
    node.nodeThread.prop=nodeThreadProps;
  formWindowClosing(null);
}//GEN-LAST:event_jButton2ActionPerformed

private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
  setVisible(false);
  dispose();
}//GEN-LAST:event_formWindowClosing

private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
  if (!IsIdInUse(Functions.intVal(jFormattedTextField1.getValue().toString())))
  {
    isOKPressed=true;
    UpdateNode();
    node.network.ActualiseIdMappingIndeces();
    formWindowClosing(null);
  }
  else
  {
    JOptionPane.showMessageDialog(null, "Selected Id already exists.", "Information", JOptionPane.INFORMATION_MESSAGE);
  }
}//GEN-LAST:event_jButton1ActionPerformed

private void jButton3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton3ActionPerformed
  try
  {
    String prop=JOptionPane.showInputDialog(this, "Property", "chef");
    if (prop!=null)
    {
      String value=JOptionPane.showInputDialog(this, "Value", "0");
      if (value!=null)
      {
        UpdateNode();
        if (!runningVisualization)
          node.prop.put(prop, value);
        else
          node.nodeThread.prop.put(prop, value);
        UpdateForm();
      }
    }
  }
  catch (Exception e)
  {
    Global.errorLog.Write("form->dialog->NodeDialog->jButton3ActionPerformed",e);
  }
}//GEN-LAST:event_jButton3ActionPerformed

private void jButton4ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton4ActionPerformed
  try
  {
    Object prop=((DefaultTableModel)jTable1.getModel()).getValueAt(jTable1.getSelectedRow(), 0);
    Object value=node.prop.get(prop);
    value=JOptionPane.showInputDialog(this, "Value", value);
    if (value!=null)
    {
      UpdateNode();
      if (!runningVisualization)
        node.prop.put(prop, value);
      else
        node.nodeThread.prop.put(prop, value);
      UpdateForm();
    }
  }
  catch (Exception e)
  {
    Global.errorLog.Write("form->dialog->NodeDialog->jButton4ActionPerformed",e);
  }
}//GEN-LAST:event_jButton4ActionPerformed

private void jButton5ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton5ActionPerformed
  try
  {
    UpdateNode();
    Object prop=((DefaultTableModel)jTable1.getModel()).getValueAt(jTable1.getSelectedRow(), 0);
    if (!runningVisualization)
      node.prop.remove(prop);
    else
      node.nodeThread.prop.remove(prop);
    UpdateForm();
  }
  catch (Exception e)
  {
    Global.errorLog.Write("form->dialog->NodeDialog->jButton5ActionPerformed",e);
  }
}//GEN-LAST:event_jButton5ActionPerformed

private void jTable1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jTable1MouseClicked
if ((evt!=null) && (evt.getClickCount()>1))
  {
    jButton4ActionPerformed(null);
  }
}//GEN-LAST:event_jTable1MouseClicked

private void jTable1MouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jTable1MouseReleased
  if ((evt!=null) && (evt.isPopupTrigger()))
  {
    jPopupMenu1.show(jTable1, evt.getX(), evt.getY());
  }
}//GEN-LAST:event_jTable1MouseReleased

private void jMenuItem1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem1ActionPerformed
  jButton3ActionPerformed(null);
}//GEN-LAST:event_jMenuItem1ActionPerformed

private void jMenuItem2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem2ActionPerformed
  jButton4ActionPerformed(null);
}//GEN-LAST:event_jMenuItem2ActionPerformed

private void jMenuItem3ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem3ActionPerformed
  jButton5ActionPerformed(null);
}//GEN-LAST:event_jMenuItem3ActionPerformed
  
  // Variables declaration - do not modify//GEN-BEGIN:variables
  public javax.swing.JButton jButton1;
  public javax.swing.JButton jButton2;
  public javax.swing.JButton jButton3;
  public javax.swing.JButton jButton4;
  public javax.swing.JButton jButton5;
  public javax.swing.JFormattedTextField jFormattedTextField1;
  public javax.swing.JLabel jLabel1;
  public javax.swing.JMenuItem jMenuItem1;
  public javax.swing.JMenuItem jMenuItem2;
  public javax.swing.JMenuItem jMenuItem3;
  public javax.swing.JPanel jPanel1;
  public javax.swing.JPanel jPanel12;
  public javax.swing.JPanel jPanel13;
  public javax.swing.JPanel jPanel2;
  public javax.swing.JPanel jPanel3;
  public javax.swing.JPanel jPanel4;
  public javax.swing.JPanel jPanel5;
  public javax.swing.JPanel jPanel6;
  public javax.swing.JPanel jPanel7;
  public javax.swing.JPanel jPanel8;
  public javax.swing.JPanel jPanel9;
  public javax.swing.JPopupMenu jPopupMenu1;
  public javax.swing.JScrollPane jScrollPane1;
  public javax.swing.JTable jTable1;
  // End of variables declaration//GEN-END:variables
  
}
