/*
 * Layout.java
 *
 * Created on tvrtok, 2008, mj 1, 22:11
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package graphics;

import java.awt.Dimension;
import java.awt.Point;
import java.util.Random;
import network.Network;
import network.Node;

/**
 *
 * @author Dur
 */
public class Layout {
  protected static int margin=10;
  protected static int defualtMaxX=1024,defaultMaxY=740;
  
  /** Creates a new instance of Layout */
  public Layout() {
  }
  
  public static int MinHorizontalPosition(Network network)
  {
    return network.CreateNode().dimension.x/2+margin;
  }
  
  public static int MaxHorizontalPosition(Network network, Dimension dimension)
  {
    return dimension.width-MinHorizontalPosition(network);
  }
  
  public static int MinVerticalPosition(Network network)
  {
    return network.CreateNode().dimension.y/2+margin;
  }
  
  public static int MaxVerticalPosition(Network network, Dimension dimension)
  {
    return dimension.height-MinVerticalPosition(network);
  }
  
  public static void RandomLayout(Network network, Dimension dimension)
  {
    if ((dimension!=null) && (network!=null))
    {
      int minX=MinHorizontalPosition(network);
      int maxX=MaxHorizontalPosition(network,dimension);
      int minY=MinVerticalPosition(network);
      int maxY=MaxVerticalPosition(network,dimension);
      Random rnd=new Random();
      for (int i=0;i<network.nodes.length;i++)
      {
        network.nodes[i].SetPosition(new Point(rnd.nextInt(maxX-minX)+minX,rnd.nextInt(maxY-minY)+minY));
      }
      StretchToWindowLayout(network,dimension);
    }
  }
  
  public static void MinimizeCrossingLayout(Network network, Dimension dimension)
  {
    
  }
  
  public static void CyclicLayout(Network network, Dimension dimension)
  {
    if ((dimension!=null) && (network!=null) && (network.nodes.length>0))
    {
      int minX=MinHorizontalPosition(network);
      int maxX=MaxHorizontalPosition(network,dimension);
      int minY=MinVerticalPosition(network);
      int maxY=MaxVerticalPosition(network,dimension);
      double radius,deltaRadius=2*Math.PI/network.nodes.length;
      int posX,posY;
      for (int i=0;i<network.nodes.length;i++)
      {
        radius=i*deltaRadius;
        posX=(int)(Math.cos(radius)*(maxX-minX)/2)+(maxX-minX)/2+minX;
        posY=(int)(Math.sin(radius)*(maxY-minY)/2)+(maxY-minY)/2+minY;
        network.nodes[i].SetPosition(new Point(posX,posY));
      }
      StretchToWindowLayout(network,dimension);
    }
  }
  
  public static void BipartiteLayout(Network network, Dimension dimension)
  {
    if ((dimension!=null) && (network!=null))
    {
      int[] set=network.MarkBipartite();
      int minX=MinHorizontalPosition(network);
      int maxX=MaxHorizontalPosition(network,dimension);
      int minY=MinVerticalPosition(network);
      int maxY=MaxVerticalPosition(network,dimension);
      int count1=0,count2=0;
      for (int i=0;i<network.nodes.length;i++)
      {
        if (set[i]==1) count1++; else count2++;
      }
      int dy1;
      if (count1!=0) 
        dy1=(maxY-minY)/count1;
      else
        dy1=dimension.getSize().height/2;
      int dy2;
      if (count2!=0)
        dy2=(maxY-minY)/count2;
      else
        dy2=dimension.getSize().height/2;
      int posY1=dy1/2;
      int posY2=dy2/2;
      for (int i=0;i<network.nodes.length;i++)
      {
        if (set[i]==1)
        {
          network.nodes[i].SetPosition(new Point(minX,posY1));
          posY1+=dy1;
        }
        else
        {
          network.nodes[i].SetPosition(new Point(maxX,posY2));
          posY2+=dy2;
        }
      }
    }
    CenterLayout(network,dimension);
  }
  
  public static void GridLayout(Network network, Dimension dimension, int width, int height)
  {
    if ((dimension!=null) && (network!=null) && (network.nodes.length>0) && (width>0) && (height>0))
    {
      int minX=MinHorizontalPosition(network);
      int maxX=MaxHorizontalPosition(network,dimension);
      int minY=MinVerticalPosition(network);
      int maxY=MaxVerticalPosition(network,dimension);
      int dx=0;
      if (width>1) 
        dx=(maxX-minX)/(width-1); 
      int dy=0;
      if (height>1) 
        dy=(maxY-minY)/(height-1); 
      
      for (int i=0;i<height;i++)
      {
        for (int j=0;j<width;j++)
        {
          network.nodes[i*width+j].SetPosition(new Point(minX+dx*j,minY+dy*i));
        }
      }
      StretchToWindowLayout(network,dimension);
    }
  }
  
  public static void SnapToGridLayout(Network network, Dimension dimension, int gridDistance)
  {
    if ((dimension!=null) && (network!=null) && (network.nodes.length>0) && (gridDistance>0))
    {
      int minX=MinHorizontalPosition(network);
      int minY=MinVerticalPosition(network);
      for (int i=0;i<network.nodes.length;i++)
      {
        Point p=network.nodes[i].position;
        p.x=(int)(Math.round((double)(p.x-minX)/gridDistance))*gridDistance+minX;
        p.y=(int)(Math.round((double)(p.y-minY)/gridDistance))*gridDistance+minY;
      }
    }
  }
  
  public static void StretchToWindowLayout(Network network, Dimension dimension)
  {
    if ((dimension!=null) && (network!=null) && (network.nodes.length>0))
    {
      int minX=MinHorizontalPosition(network);
      int maxX=MaxHorizontalPosition(network,dimension);
      int minY=MinVerticalPosition(network);
      int maxY=MaxVerticalPosition(network,dimension);
      int actMinX=Integer.MAX_VALUE,actMaxX=Integer.MIN_VALUE,actMinY=Integer.MAX_VALUE,actMaxY=Integer.MIN_VALUE;
      for (int i=0;i<network.nodes.length;i++)
      {
        if (network.nodes[i].position.x<actMinX) actMinX=network.nodes[i].position.x;
        if (network.nodes[i].position.x>actMaxX) actMaxX=network.nodes[i].position.x;
        if (network.nodes[i].position.y<actMinY) actMinY=network.nodes[i].position.y;
        if (network.nodes[i].position.y>actMaxY) actMaxY=network.nodes[i].position.y;
      }
      
      Node node=network.CreateNode();
      int actWidth=actMaxX-actMinX;
      int actHeight=actMaxY-actMinY;
      if (((double)(maxX-minX)/actWidth)<((double)(maxY-minY)/actHeight))
        actHeight=(int)((maxY-minY)/((double)(maxX-minX)/actWidth));
      else
        actWidth=(int)((maxX-minX)/((double)(maxY-minY)/actHeight));
      int posX,posY;
      for (int i=0;i<network.nodes.length;i++)
      {
        if (actWidth!=0)
          posX=(int)(minX+(network.nodes[i].position.x-actMinX)*((double)(maxX-minX)/actWidth));
        else
          posX=dimension.width/2;
        if (actHeight!=0)
          posY=(int)(minY+(network.nodes[i].position.y-actMinY)*((double)(maxY-minY)/actHeight));
        else
          posY=dimension.height/2;
        network.nodes[i].SetPosition(new Point(posX,posY));
      }
      CenterLayout(network,dimension);
    }
  }
  
  public static void CenterLayout(Network network, Dimension dimension)
  {
    if ((dimension!=null) && (network!=null) && (network.nodes.length>0))
    {
      int minX=MinHorizontalPosition(network);
      int maxX=MaxHorizontalPosition(network,dimension);
      int minY=MinVerticalPosition(network);
      int maxY=MaxVerticalPosition(network,dimension);
      int actMinX=Integer.MAX_VALUE,actMaxX=Integer.MIN_VALUE,actMinY=Integer.MAX_VALUE,actMaxY=Integer.MIN_VALUE;
      for (int i=0;i<network.nodes.length;i++)
      {
        if (network.nodes[i].position.x<actMinX) actMinX=network.nodes[i].position.x;
        if (network.nodes[i].position.x>actMaxX) actMaxX=network.nodes[i].position.x;
        if (network.nodes[i].position.y<actMinY) actMinY=network.nodes[i].position.y;
        if (network.nodes[i].position.y>actMaxY) actMaxY=network.nodes[i].position.y;
      }
      
      Node node=network.CreateNode();
      int actWidth=actMaxX-actMinX;
      int actHeight=actMaxY-actMinY;
      
      int posX,posY;
      for (int i=0;i<network.nodes.length;i++)
      {
        if (actWidth!=0)
          posX=(int)(minX+network.nodes[i].position.x-actMinX+(maxX-minX-actWidth)/2);
        else
          posX=dimension.width/2;
        if (actHeight!=0)
          posY=(int)(minY+network.nodes[i].position.y-actMinY+(maxY-minY-actHeight)/2);
        else
          posY=dimension.height/2;
        network.nodes[i].SetPosition(new Point(posX,posY));
      }
    }
  }
  
}
