/*
 * StandardVisualization.java
 *
 * Created on Streda, 2008, mj 7, 21:47
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package graphics;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Point;
import java.awt.Stroke;
import java.io.Serializable;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.util.Iterator;
import java.util.Map;
import network.Link;
import network.LinkMessage;
import network.Node;
import other.AssociativeArray;
import other.Functions;
import player.Player;

/**
 *
 * @author Dur
 */
public class StandardVisualization extends Visualization implements Serializable
{
  protected transient Stroke dashedStroke=new BasicStroke(1, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 10, new float[] {10,10}, 0);
  protected transient Stroke normalStroke=new BasicStroke();
  protected transient MessageDigest sha1=null;
  
  /**
   * Creates a new instance of StandardVisualization
   */
  public StandardVisualization() 
  {
    super();
    graphics.SetColor(Color.BLACK);
    try
    {
      sha1=MessageDigest.getInstance("SHA-1");
    }
    catch (Exception e){}
  }
  
  protected Color GetColorFromString(String s)
  {
    if (s!=null)
    {
      if (sha1!=null)
      {
        sha1.update(s.getBytes());
        Color c=Color.decode("#"+new BigInteger(1, sha1.digest()).toString(16).substring(1, 7));
        return c;
      }
      int num=s.hashCode();
      if (num<0)
        num=-num;
      Color c=new Color(num);
      return c;
    }
    return Color.WHITE;
  }
  
  public void DrawStaticNode(Node node)
  {
    Color c=Color.WHITE;
    Object sameColor=visualizationData.properties.get("same_color");
    if ((sameColor!=null) && (!sameColor.equals("")))
    {
      Object o=node.prop.get(sameColor);
      if (o!=null)
        c=GetColorFromString(o.toString());
    }
    else
    {
      Iterator<AssociativeArray<Object, Object>> it=visualizationData.nodeBgColors.iterator();
      boolean found=false;
      while (it.hasNext() && (!found))
      {
        AssociativeArray<Object, Object> o=it.next();
        Object prop=node.prop.get(o.get("property"));
        if ((prop!=null) && (prop.equals(o.get("equals_to"))))
        {
          found=true;
          c=(Color)o.get("color");
        }
      }
    }
    Object o;
    if ((o=node.prop.get(visualizationData.properties.get("node_text")))==null)
    {
      o="";
    }
    graphics.SetFillColor(c);
    graphics.SetColor(Color.BLACK);
    graphics.SetStroke(normalStroke);
    graphics.DrawLabelledEllipse(node.position,node.dimension,o.toString());
  }
  
  public void DrawStaticSelectedNode(Node node)
  {
    Color c=Color.WHITE;
    Object sameColor=visualizationData.properties.get("same_color");
    if ((sameColor!=null) && (!sameColor.equals("")))
    {
      Object o=node.prop.get(sameColor);
      if (o!=null)
        c=GetColorFromString(o.toString());
    }
    else
    {
      Iterator<AssociativeArray<Object, Object>> it=visualizationData.nodeBgColors.iterator();
      boolean found=false;
      while (it.hasNext() && (!found))
      {
        AssociativeArray<Object, Object> o=it.next();
        Object prop=node.prop.get(o.get("property"));
        if ((prop!=null) && (prop.equals(o.get("equals_to"))))
        {
          found=true;
          c=(Color)o.get("color");
        }
      }
    }
    Object o;
    if ((o=node.prop.get(visualizationData.properties.get("node_text")))==null)
    {
      o="";
    }
    graphics.SetFillColor(c);
    graphics.SetStroke(new BasicStroke(3));
    graphics.SetColor(Color.GRAY);
    graphics.DrawLabelledEllipse(node.position,node.dimension,o.toString());
  }
  
  public void DrawStaticLink(Link link)
  {
    boolean active=Functions.BooleanValDef(link.prop.get("active").toString(), true);
    if (active)
      graphics.SetStroke(normalStroke);
    else
      graphics.SetStroke(dashedStroke);
    graphics.SetColor(Color.BLACK);
    graphics.DrawLabelledLine(link.GetFirstNode().position,link.GetSecondNode().position);
  }
  
  public void DrawNode(Player player, Node node)
  {
    Map<Object, Object> properties=node.prop;
    if (node.nodeThread!=null)
    {
      properties=node.nodeThread.prop;
    }
    Color c=Color.WHITE;
    Object sameColor=visualizationData.properties.get("same_color");
    if ((sameColor!=null) && (!sameColor.equals("")))
    {
      Object o=properties.get(sameColor);
      if (o!=null)
        c=GetColorFromString(o.toString());
    }
    else
    {
      Iterator<AssociativeArray<Object, Object>> it=visualizationData.nodeBgColors.iterator();
      boolean found=false;
      while (it.hasNext() && (!found))
      {
        AssociativeArray<Object, Object> o=it.next();
        Object prop=properties.get(o.get("property"));
        if ((prop!=null) && (prop.equals(o.get("equals_to"))))
        {
          found=true;
          c=(Color)o.get("color");
        }
      }
    }
    Object o;
    if ((o=properties.get(visualizationData.properties.get("node_text")))==null)
    {
      o="";
    }
    graphics.SetFillColor(c);
    graphics.SetStroke(normalStroke);
    graphics.SetColor(Color.BLACK);
    graphics.DrawLabelledEllipse(node.position,node.dimension,o.toString());
  }
  
  public void DrawSelectedNode(Player player, Node node)
  {
    Map<Object, Object> properties=node.prop;
    if (node.nodeThread!=null)
    {
      properties=node.nodeThread.prop;
    }
    Color c=Color.WHITE;
    Object sameColor=visualizationData.properties.get("same_color");
    if ((sameColor!=null) && (!sameColor.equals("")))
    {
      Object o=properties.get(sameColor);
      if (o!=null)
        c=GetColorFromString(o.toString());
    }
    else
    {
      Iterator<AssociativeArray<Object, Object>> it=visualizationData.nodeBgColors.iterator();
      boolean found=false;
      while (it.hasNext() && (!found))
      {
        AssociativeArray<Object, Object> o=it.next();
        Object prop=properties.get(o.get("property"));
        if ((prop!=null) && (prop.equals(o.get("equals_to"))))
        {
          found=true;
          c=(Color)o.get("color");
        }
      }
    }
    Object o;
    if ((o=properties.get(visualizationData.properties.get("node_text")))==null)
    {
      o="";
    }
    graphics.SetFillColor(c);
    graphics.SetStroke(new BasicStroke(3));
    graphics.SetColor(Color.GRAY);
    graphics.DrawLabelledEllipse(node.position,node.dimension,o.toString());
  }
  
  public void DrawLink(Player player, Link link)
  {
    boolean active=Functions.BooleanValDef(link.prop.get("active").toString(), true);
    if (!active)
    {
      int activationTimer=Functions.intVal(link.prop.get("activationTimer").toString());
      if ((activationTimer>0) && (activationTimer<=player.GetRunningTime()))
        active=true;
    }
    if (active)
      graphics.SetStroke(normalStroke);
    else
      graphics.SetStroke(dashedStroke);
    graphics.SetColor(Color.BLACK);
    graphics.DrawLabelledLine(link.GetFirstNode().position,link.GetSecondNode().position);
  }
  
  public void DrawSelectedLink(Player player, Link link)
  {
    boolean active=Functions.BooleanValDef(link.prop.get("active").toString(), true);
    if (!active)
    {
      int activationTimer=Functions.intVal(link.prop.get("activationTimer").toString());
      if ((activationTimer>0) && (activationTimer<=player.GetRunningTime()))
        active=true;
    }
    graphics.SetStroke(new BasicStroke(3));
    graphics.SetColor(Color.GRAY);
    graphics.DrawLabelledLine(link.GetFirstNode().position,link.GetSecondNode().position);
  }
  
  public void DrawMessage(Player player, LinkMessage mes)
  {
    if (!visualizationData.notVisualizingMessageTypes.containsKey(mes.message.GetType()))
    {
      float ratio=mes.GetProgress();
      if (ratio>1)
        ratio=1;
      int x,y;
      x=mes.recipient.position.x-mes.sender.position.x;
      x=mes.sender.position.x+Math.round(((float)x)*ratio);
      y=mes.recipient.position.y-mes.sender.position.y;
      y=mes.sender.position.y+Math.round(((float)y)*ratio);
      graphics.SetFillColor(mes.message.GetBackgroundColor());
      graphics.SetStroke(normalStroke);
      graphics.SetColor(Color.BLACK);
      graphics.DrawLabelledEllipse(new Point(x, y), LinkMessage.dimension);
    }
  }
  
  public void DrawSelectedMessage(Player player, LinkMessage mes)
  {
    if (!visualizationData.notVisualizingMessageTypes.containsKey(mes.message.GetType()))
    {
      float ratio=mes.GetProgress();
      if (ratio>1)
        ratio=1;
      int x,y;
      x=mes.recipient.position.x-mes.sender.position.x;
      x=mes.sender.position.x+Math.round(((float)x)*ratio);
      y=mes.recipient.position.y-mes.sender.position.y;
      y=mes.sender.position.y+Math.round(((float)y)*ratio);
      graphics.SetFillColor(mes.message.GetBackgroundColor());
      graphics.SetStroke(new BasicStroke(3));
      graphics.SetColor(Color.GRAY);
      graphics.DrawLabelledEllipse(new Point(x, y), LinkMessage.dimension);
    }
  }
  
  public void Repair()
  {
    dashedStroke=new BasicStroke(1, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 10, new float[] {10,10}, 0);
    normalStroke=new BasicStroke();
    try
    {
      sha1=MessageDigest.getInstance("SHA-1");
    }
    catch (Exception e){}
  }

  
}
