/*
 * Visualization.java
 *
 * Created on Pondelok, 2008, aprl 14, 18:01
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package graphics;

import java.awt.BasicStroke;
import java.awt.Color;
import java.io.Serializable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.TreeMap;
import network.Link;
import network.LinkMessage;
import network.Network;
import network.Node;
import player.Player;

/**
 *
 * @author Dur
 */
public abstract class Visualization implements Serializable
{
  protected Graphics graphics;
  protected StandardVisualizationData visualizationData;
  
  /** Creates a new instance of Visualization */
  public Visualization() {
    graphics=new Graphics();
  }
  
  public void DrawStaticNetwork(Network network,java.awt.Graphics g)
  {
    visualizationData=network.GetAlgorithm().GetActivatedVisualizationData();
    graphics.SetGraphics(g);
    graphics.Clear();
    Node[] nodes=network.nodes;
    for (int i=0;i<network.links.length;i++)
    {
      DrawStaticLink(network.links[i]);
    }
    for (int i=0;i<nodes.length;i++)
    {
      DrawStaticNode(nodes[i]);
    }
  }
  
  public void DrawNetwork(Player player,java.awt.Graphics g)
  {
    visualizationData=player.network.GetAlgorithm().GetActivatedVisualizationData();
    graphics.SetGraphics(g);
    graphics.Clear();
    Node[] nodes=player.network.nodes;
    for (int i=0;i<player.network.links.length;i++)
    {
      DrawLink(player, player.network.links[i]);
    }
    if (player.linksThread!=null)
    {
      TreeMap<Long, LinkedList<LinkMessage>> mapMes=player.linksThread.activeMessages;
      synchronized (mapMes)
      {
        Iterator<LinkedList<LinkMessage>> itMap=mapMes.values().iterator();
        while (itMap.hasNext())
        {
          LinkedList<LinkMessage> listMes=itMap.next();
          synchronized (listMes)
          {
            if (listMes!=null)
            {
              Iterator<LinkMessage> itList=listMes.iterator();
              while (itList.hasNext())
              {
                LinkMessage mes=itList.next();
                DrawMessage(player, mes);
              }
            }
          }
        }
      }
    }
    for (int i=0;i<nodes.length;i++)
    {
      DrawNode(player,nodes[i]);
    }
  }
  
  public abstract void DrawStaticNode(Node node);
  
  public abstract void DrawStaticLink(Link link);
  
  public abstract void DrawNode(Player player, Node node);
  
  public abstract void DrawSelectedNode(Player player, Node node);
  
  public abstract void DrawLink(Player player, Link link);
  
  public abstract void DrawSelectedLink(Player player, Link link);
  
  public abstract void DrawMessage(Player player, LinkMessage mes);
  
  public abstract void DrawSelectedMessage(Player player, LinkMessage mes);
  
  public void DrawStaticSelectedLink(Link link)
  {
    graphics.SetColor(Color.GRAY);
    graphics.SetStroke(new BasicStroke(3));
    graphics.DrawLabelledLine(link.GetFirstNode().position,link.GetSecondNode().position);
    DrawStaticNode(link.GetFirstNode());
    DrawStaticNode(link.GetSecondNode());
  }
  
  public abstract void DrawStaticSelectedNode(Node node);
  
  public Graphics GetGraphics()
  {
    return graphics;
  }
  
}
