/*
 * Link.java
 *
 * Created on Utorok, 2008, aprl 1, 11:01
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package network;

import form.dialog.LinkDialog;
import java.awt.Frame;
import java.awt.Point;
import java.io.Serializable;
import java.util.LinkedList;
import java.util.Random;
import other.AssociativeArray;
import other.Functions;
import other.IDialog;

/**
 *
 * @author Dur
 */
public class Link implements Serializable
{
  private int linkArea=10;
  
  protected Node node1,node2;
  protected transient LinkedList<LinkMessage> messageBuf1=new LinkedList<LinkMessage>();
  protected transient LinkedList<LinkMessage> messageBuf2=new LinkedList<LinkMessage>();
  public Constraint[] constraints=new Constraint[0];
  public AssociativeArray<Object, Object> prop=new AssociativeArray<Object, Object>();
  
  
  /** Creates a new instance of Link */
  public Link(Node n1,Node n2) {
    node1=n1;
    node2=n2;
    Random rnd=new Random();
    prop.put("transferSpeed", rnd.nextInt(2000)+2000);
    prop.put("activationTimer", new Integer(0));
    prop.put("active", new Boolean(true));
  }
  
  public Node GetFirstNode()
  {
    return node1;
  }
  
  public Node GetSecondNode()
  {
    return node2;
  }
  
  public Node GetOtherNode(Node n1)
  {
    if (n1!=null)
    {
      if (node1==n1) return node2;
      if (node2==n1) return node1;
    }
    return null;
  }
  
  public Node GetNodeById(Object id)
  {
    try
    {
      if (node1.prop.get("id").equals(id))
        return node1;
      if (node2.prop.get("id").equals(id))
        return node2;
    }
    catch (Exception e){}
    return null;
  }
  
  public void AddConstraint(Constraint c)
  {
    Constraint[] newCons=new Constraint[constraints.length+1];
    System.arraycopy(constraints, 0, newCons, 0, constraints.length);
    newCons[newCons.length-1]=c;
    constraints=newCons;
  }
  
  public void DeleteConstraint(int constraintIndex)
  {
    if ((constraintIndex>=0) && (constraintIndex<constraints.length))
    {
      Constraint[] newCons=new Constraint[constraints.length-1];
      int pos=0;
      for (int i=0;i<constraints.length;i++)
      {
        if (i!=constraintIndex)
        {
          newCons[pos]=constraints[i];
          pos++;
        }
      }
      constraints=newCons;
    }
  }
  
  public boolean IsAtPosition(Point p)
  {
    if ((node1!=null) && (node2!=null))
    {
      Point p1,p2,p3;
      p1=node1.position;
      p2=node2.position;
      if (((p1.x<=p.x) && (p.x<=p2.x)) || ((p2.x<=p.x) && (p.x<=p1.x)))
      {
        if (p1.x!=p2.x) 
        {
          p3=new Point(p.x,(int)(p1.y+(p.x-p1.x)*((double)(p2.y-p1.y)/(double)(p2.x-p1.x))));
          if (Math.abs(p.y-p3.y)<=linkArea) return true;
        }
        else
        {
          if (Math.abs(p1.y-p.y)<=linkArea) return true;
        }
      }
      else if (((p1.y<=p.y) && (p.y<=p2.y)) || ((p2.y<=p.y) && (p.y<=p1.y)))
      {
        if (p1.y!=p2.y) 
        {
          p3=new Point((int)(p1.x+(p.y-p1.y)*((double)(p2.x-p1.x)/(double)(p2.y-p1.y))),p1.y);
          if (Math.abs(p.x-p3.x)<=linkArea) return true;
        }
        else
        {
          if (Math.abs(p1.x-p.x)<=linkArea) return true;
        }
      }
    }
    return false;
  }
  
  public IDialog CreatePropertiesDialog(Frame parent, boolean modal)
  {
    return new LinkDialog(this,parent,modal);
  }
  
  public void PrepareForVisualization()
  {
    synchronized (messageBuf1)
    {
      messageBuf1.clear();
    }
    synchronized (messageBuf2)
    {
      messageBuf2.clear();
    }
  }
  
  public LinkedList<LinkMessage> GetMessageBuffer(Node node)
  {
    if (node==node1)
    {
      return messageBuf1;
    }
    if (node==node2)
    {
      return messageBuf2;
    }
    return null;
  }
  
  public int GetNumberOfMessages(Node node)
  {
    if (node==node1)
    {
      int num=0;
      synchronized (messageBuf1)
      {
        num=messageBuf1.size();
      }
      return num;
    }
    if (node==node2)
    {
      int num=0;
      synchronized (messageBuf2)
      {
        num=messageBuf2.size();
      }
      return num;
    }
    return -1;
  }
  
  public LinkMessage GetLinkMessage(Node node, int index)
  {
    LinkMessage res=null;
    if (node==node1)
    {
      synchronized (messageBuf1)
      {
        try
        {
          res=messageBuf1.get(index);
        }
        catch (Exception e){}
      }
      return res;
    }
    if (node==node2)
    {
      synchronized (messageBuf2)
      {
        try
        {
          res=messageBuf2.get(index);
        }
        catch (Exception e){}
      }
      return res;
    }
    return null;
  }
  
  public LinkMessage RemoveFirstLinkMessage(Node node)
  {
    LinkMessage res=null;
    if (node==node1)
    {
      synchronized (messageBuf1)
      {
        try
        {
          res=messageBuf1.remove();
        }
        catch (Exception e){}
      }
      return res;
    }
    if (node==node2)
    {
      synchronized (messageBuf2)
      {
        try
        {
          res=messageBuf2.remove();
        }
        catch (Exception e){}
      }
      return res;
    }
    return null;
  }
  
  public void AppendMessage(LinkMessage message)
  {
    if (message.sender==node1)
    {
      synchronized (messageBuf2)
      {
        messageBuf2.add(message);
      }
    }
    if (message.sender==node2)
    {
      synchronized (messageBuf1)
      {
        messageBuf1.add(message);
      }
    }
  }
  
  @Override
  public int hashCode() 
  {
    return Functions.intVal(node1.prop.get("id").toString())+Functions.intVal(node2.prop.get("id").toString());
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    final Link other = (Link) obj;
    if (other==this)
      return true;
    if (other.node1.prop.get("id").equals(node1.prop.get("id")))
    {
      if (other.node2.prop.get("id").equals(node2.prop.get("id")))
        return true;
    }
    return false;
  }
  
  public void Repair()
  {
    messageBuf1=new LinkedList<LinkMessage>();
    messageBuf2=new LinkedList<LinkMessage>();
  }
 
}
