/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package network;

import java.awt.Color;
import java.awt.Point;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import other.Functions;
import other.Global;
import player.Player;

/**
 *
 * @author Dur
 */
public class LinkMessage implements Comparable<LinkMessage>{
  protected LinkedList<Link> linkPath=new LinkedList<Link>();
  public Node sender;
  public Node recipient;
  public Link link;
  public Message message;
  public long creationMilisec;
  public long completionMilisec;
  public Player player;
  public int numLinkPassed=0;
  public static Point dimension=new Point(10, 10);
  
  public LinkMessage(Player player, Node sender, Link link, Message message)
  {
    this.player=player;
    this.sender=sender;
    this.recipient=link.GetOtherNode(sender);
    this.link=link;
    this.message=message;
    creationMilisec=player.GetRunningTime();
    this.completionMilisec=GetCompletionMilisec(link, creationMilisec);
    Color c=sender.network.GetAlgorithm().GetActivatedVisualizationData().messageTypesBgColors.get(message.type);
    if (c!=null)
      message.SetBackgroundColor(c);
  }
  
  public LinkMessage(Player player, Node sender, Node recipient, Message message)
  {
    this.player=player;
    linkPath=GetLinkPath(sender, recipient);
    this.sender=sender;
    this.link=linkPath.remove();
    this.recipient=link.GetOtherNode(sender);
    this.message=message;
    creationMilisec=player.GetRunningTime();
    this.completionMilisec=GetCompletionMilisec(link, creationMilisec);
    Color c=sender.network.GetAlgorithm().GetActivatedVisualizationData().messageTypesBgColors.get(message.type);
    if (c!=null)
      message.SetBackgroundColor(c);
  }
  
  public long GetCompletionMilisec(Link link, long creationMilisec)
  {
    try
    {
      int ms=Functions.intVal(link.prop.get("transferSpeed").toString());
      return ms+creationMilisec;
    }
    catch (Exception e){}
    return player.GetRunningTime();
  }
  
  protected LinkedList<Link> GetLinkPath(Node nodeFrom, Node nodeTo)
  {
    try
    {
      LinkedList<Node> front=new LinkedList<Node>();
      front.add(nodeFrom);
      HashSet<Object> visitedNodeIds=new HashSet<Object>();
      visitedNodeIds.add(nodeFrom.prop.get("id"));
      HashMap<Object, Link> linkToNode=new HashMap<Object, Link>();
      Node node1;
      Node node2;
      while (front.size()>0)
      {
        node1=front.remove();
        for (int i=0;i<node1.links.length;i++)
        {
          node2=node1.links[i].GetOtherNode(node1);
          Object id=node2.prop.get("id");
          if (!visitedNodeIds.contains(id))
          {
            visitedNodeIds.add(id);
            linkToNode.put(id, node1.links[i]);
            front.add(node2);
            if (nodeTo==node2)
            {
              LinkedList<Link> list=new LinkedList<Link>();
              Link l;
              do 
              {
                l=linkToNode.get(node2.prop.get("id"));
                list.addFirst(l);
                node2=l.GetOtherNode(node2);
              } while (node2!=nodeFrom);
              return list;
            }
          }
        }
      }
    }
    catch (Exception e)
    {
      Global.errorLog.Write("Error: LinkMessage->GetLinkPath", e);
    }
    return null;
  }
  
  public LinkMessage GetNextLinkMessage()
  {
    if (linkPath.size()>0)
    {
      LinkMessage newMessage=new LinkMessage(player, recipient, linkPath.remove(), message);
      newMessage.linkPath=new LinkedList<Link>(linkPath);
      newMessage.numLinkPassed=numLinkPassed+1;
      return newMessage;
    }
    return null;
  }
  
  public float GetProgress()
  {
    float res=completionMilisec-creationMilisec;
    long now=player.GetRunningTime();
    if (res!=0)
    {
      return (now-creationMilisec)/res;
    }
    return 1;
  }

  @Override
  public int hashCode() {
    return (int)(completionMilisec % (((long)Integer.MAX_VALUE)+1));
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    final LinkMessage other = (LinkMessage) obj;
    if (this.completionMilisec != other.completionMilisec) {
      return false;
    }
    return true;
  }
  
  public int compareTo(LinkMessage o)
  {
    return (completionMilisec<o.completionMilisec ? -1 : (completionMilisec==o.completionMilisec ? 0 : 1));
  }
  
  public static void Repair()
  {
    dimension=new Point(10, 10);
  }
  
}
