/*
 * Network.java
 *
 * Created on Utorok, 2008, aprl 1, 12:48
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package network;

import algorithm.Algorithm;
import form.NetworkEditorForm;
import form.VisualizationForm;
import graphics.Layout;
import java.awt.Dimension;
import java.awt.Point;
import java.io.Serializable;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Random;
import javax.swing.JFrame;
import other.AssociativeArray;
import other.Functions;
import player.Player;

/**
 *
 * @author Dur
 */
public class Network implements Serializable
{
  protected String name="new_network";
  protected Algorithm algorithm;
  protected HashMap<Integer,Integer> idMappingIndeces=new HashMap<Integer, Integer>();
  public Node[] nodes;
  public Link[] links;
  public AssociativeArray<Object, Object> prop=new AssociativeArray<Object, Object>();
  
  public Node CreateNode()
  {
    return new Node(this);
  }
  
  public Link CreateLink(Node n1,Node n2)
  {
    return new Link(n1,n2);
  }
  
  /** Creates a new instance of Network */
  public Network(Algorithm algorithm) {
    this.algorithm=algorithm;
    nodes=new Node[0];
    links=new Link[0];
    prop.put("networkType", "asynchronous");
  }
  
  public String GetName()
  {
    return name;
  }
  
  public void SetName(String name)
  {
    this.name=name;
  }
  
  public Node AddNode()
  {
    Node[] ns=new Node[nodes.length+1];
    System.arraycopy(nodes,0,ns,0,nodes.length);
    ns[ns.length-1]=CreateNode();
    nodes=ns;
    ActualiseIdMappingIndeces();
    return nodes[nodes.length-1];
  }
  
  public void DeleteNode(Node node)
  {
    if (node!=null)
    {
      while (node.links.length!=0)
      {
        DeleteLink(node.links[0]);
      }
      
      int pos=0;
      for (int i=0;i<nodes.length;i++)
      {
        if (nodes[i]!=node)
        {
          nodes[pos]=nodes[i];
          pos++;
        }
      }
      Node[] newNodes=new Node[pos];
      System.arraycopy(nodes,0,newNodes,0,pos);
      nodes=newNodes;
      ActualiseIdMappingIndeces();
    }
  }
  
  public void ActualiseIdMappingIndeces()
  {
    HashMap<Integer, Integer> map=new HashMap<Integer, Integer>();
    for (int i=0;i<nodes.length;i++)
    {
      map.put(Functions.intVal(nodes[i].prop.get("id").toString()), i);
    }
    idMappingIndeces=map;
  }
  
  public int GetNodeIndex(Node node)
  {
    Integer index=idMappingIndeces.get(Functions.intVal(node.prop.get("id").toString()));
    if (index!=null)
    {
      return index;
    }
    return -1;
  }
  
  public Node GetNodeById(int id)
  {
    for (int i=0;i<nodes.length;i++)
    {
      if (Functions.intVal(nodes[i].prop.get("id").toString())==id)
      {
        return nodes[i];
      }
    }
    return null;
  }
  
  public Link AddLink(Node node1, Node node2)
  {
    if ((node1!=node2) && (!LinkExists(node1,node2)))
    {
      Link[] ls=new Link[links.length+1];
      System.arraycopy(links,0,ls,0,links.length);
      ls[ls.length-1]=CreateLink(node1,node2);
      links=ls;
      node1.AddLink(links[links.length-1]);
      node2.AddLink(links[links.length-1]);
      return links[links.length-1];
    }
    return null;
  }
  
  public void DeleteLink(Link link)
  {
    if (link!=null)
    {
      Node node=link.GetFirstNode();
      if (node!=null)
      {
        node.DeleteLink(link);
      }
      node=link.GetSecondNode();
      if (node!=null)
      {
        node.DeleteLink(link);
      }
      
      int pos=0;
      for (int i=0;i<links.length;i++)
      {
        if (links[i]!=link)
        {
          links[pos]=links[i];
          pos++;
        }
      }
      Link[] newLinks=new Link[pos];
      System.arraycopy(links,0,newLinks,0,pos);
      links=newLinks;
    }
  }
  
  public boolean LinkExists(Node node1, Node node2)
  {
    for (int i=0;i<node1.links.length;i++)
    {
      if (node1.links[i].GetOtherNode(node1)==node2) return true;
    }
    return false;
  }
  
  public void Clear()
  {
    nodes=new Node[0];
    links=new Link[0];
  }
  
  public void OpenEditorForm(JFrame caller)
  {
    NetworkEditorForm frm=new NetworkEditorForm(caller,this);
    frm.setVisible(true);
  }
  
  public void OpenVisualizationForm(JFrame caller)
  {
    VisualizationForm frm=new VisualizationForm(caller,this);
    frm.setVisible(true);
  }
  
  public Algorithm GetAlgorithm()
  {
    return algorithm;
  }
  
  public Node GetNodeAtPosition(Point p)
  {
    for (int i=0;i<nodes.length;i++)
    {
      if (nodes[i].IsAtPosition(p))
      {
        return nodes[i];
      }
    }
    return null;
  }
  
  public Link GetLinkAtPosition(Point p)
  {
    for (int i=0;i<links.length;i++)
    {
      if (links[i].IsAtPosition(p))
      {
        return links[i];
      }
    }
    return null;
  }
  
  public boolean IsWholeGraphVisible(Dimension dimension)
  {
    if (dimension!=null)
    {
      for (int i=0;i<nodes.length;i++)
      {
        if ((nodes[i].position.x<0) || (nodes[i].position.y<0)) return false;
        if ((nodes[i].position.x>dimension.width) || (nodes[i].position.y>dimension.height)) return false;
      }
    }
    return true;
  }
  
  public int NextFreeNodeId()
  {
    boolean[] occupied=new boolean[nodes.length+1];
    for (int i=0;i<occupied.length;i++)
    {
      occupied[i]=false;
    }
    int id;
    for (int i=0;i<nodes.length;i++)
    {
      if (nodes[i]!=null)
      {
        Object o=nodes[i].prop.get("id");
        if (o!=null)
        {
          id=Functions.intVal(o.toString());
          if ((id>=0) && (id<occupied.length))
            occupied[id]=true;
        }
      }
    }
    for (int i=0;i<occupied.length;i++)
    {
      if (!occupied[i])
        return i;
    }
    //following statement should never be executed
    return -1;
  }
  
  public boolean IsNodeIdInUse(int id)
  {
    int id2;
    for (int i=0;i<nodes.length;i++)
    {
      id2=Functions.intVal(nodes[i].prop.get("id").toString());
      if (id2==id)
        return true;
    }
    return false;
  }
  
  public void CreateRandomGraph(int node, int link, Dimension dimension)
  {
    Clear();
    nodes=new Node[node];
    for (int i=0;i<node;i++)
    {
      nodes[i]=CreateNode();
    }
    
    if (link>(node*(node-1)/2)) link=node*(node-1)/2;
    Random rnd=new Random();
    int n1,n2;
    for (int i=0;i<link;i++)
    {
      do
        n1=rnd.nextInt(node);
      while (nodes[n1].links.length>=(node-1));
      do
        n2=rnd.nextInt(node);
      while (LinkExists(nodes[n1],nodes[n2]) || (n1==n2));
      AddLink(nodes[n1],nodes[n2]);
    }
    ShuffleLinks();
    ActualiseIdMappingIndeces();
    Layout.RandomLayout(this,dimension);
  }
  
  public void CreatePlanarGraph(int node, int link, Dimension dimension)
  {
    
  }
  
  public void CreateCompleteGraph(int node, Dimension dimension)
  {
    Clear();
    nodes=new Node[node];
    for (int i=0;i<node;i++)
    {
      nodes[i]=CreateNode();
    }
    
    for (int i=0;i<node;i++)
    {
      for (int j=0;j<node;j++)
      {
        if (i!=j) AddLink(nodes[i],nodes[j]);
      }
    }
    ShuffleLinks();
    Layout.CyclicLayout(this,dimension);
    ActualiseIdMappingIndeces();
  }
  
  protected void MarkBipartite(Node node, int color, int[] result)
  {
    if (node!=null)
    {
      int index=GetNodeIndex(node);
      if (result[index]==0)
      {
        result[index]=color;
        if (color==1) color=2; else color=1;
        for (int i=0;i<node.links.length;i++)
        {
          MarkBipartite(node.links[i].GetOtherNode(node),color,result); 
        }
      }
    }
  }
  
  //returns array, where [i]==1 || [i]==2. That determines which Node is which color colored with. 
  public int[] MarkBipartite()
  {
    int[] result=new int[nodes.length];
    for (int i=0;i<nodes.length;i++)
    {
      result[i]=0;
    }
    for (int i=0;i<nodes.length;i++)
    {
      if (result[i]==0)
      {
        MarkBipartite(nodes[i],1,result);
      }
    }
    return result;
  }
  
  public void CreateBipartiteGraph(int node1, int node2, int link, Dimension dimension)
  {
    Clear();
    nodes=new Node[node1+node2];
    for (int i=0;i<(node1+node2);i++)
    {
      nodes[i]=CreateNode();
    }
    
    if (link>(node1*node2)) link=node1*node2;
    Random rnd=new Random();
    int n1,n2;
    for (int i=0;i<link;i++)
    {
      do
        n1=rnd.nextInt(node1);
      while (nodes[n1].links.length>=node2);
      do
        n2=rnd.nextInt(node2)+node1;
      while (LinkExists(nodes[n1],nodes[n2]));
      AddLink(nodes[n1],nodes[n2]);
    }
    ShuffleLinks();
    ActualiseIdMappingIndeces();
    Layout.BipartiteLayout(this,dimension);
  }
  
  public void CreateCompleteBipartiteGraph(int node1, int node2, Dimension dimension)
  {
    Clear();
    nodes=new Node[node1+node2];
    for (int i=0;i<(node1+node2);i++)
    {
      nodes[i]=CreateNode();
    }
    
    for (int i=0;i<node1;i++)
    {
      for (int j=0;j<node2;j++)
      {
        AddLink(nodes[i],nodes[j+node1]);
      }
    }
    ShuffleLinks();
    ActualiseIdMappingIndeces();
    Layout.BipartiteLayout(this,dimension);
  }
  
  public void CreateCyclicGraph(int node, Dimension dimension)
  {
    Clear();
    nodes=new Node[node];
    for (int i=0;i<node;i++)
    {
      nodes[i]=CreateNode();
    }
    
    if (node>1)
    {
      for (int i=0;i<(node-1);i++)
      {
        AddLink(nodes[i],nodes[i+1]);
      }
      AddLink(nodes[0],nodes[node-1]);
    }
    ActualiseIdMappingIndeces();
    ShuffleLinks();
    Layout.CyclicLayout(this,dimension);
  }
  
  public void CreateGridGraph(int width, int height, Dimension dimension)
  {
    Clear();
    nodes=new Node[width*height];
    for (int i=0;i<nodes.length;i++)
    {
      nodes[i]=CreateNode();
    }
    
    for (int i=0;i<height;i++)
    {
      for (int j=0;j<(width-1);j++)
      {
        AddLink(nodes[i*width+j],nodes[i*width+j+1]);
      }
    }
    for (int i=0;i<(height-1);i++)
    {
      for (int j=0;j<width;j++)
      {
        AddLink(nodes[i*width+j],nodes[i*width+j+width]);
      }
    }
    ActualiseIdMappingIndeces();
    ShuffleLinks();
    Layout.GridLayout(this,dimension,width,height);
  }
  
  public void CreateTreeGraph(int nodeCount, int depth, Dimension d)
  {
    Clear();
    if (nodeCount<depth)
      depth=nodeCount;
    if (depth<2)
      depth=2;
    int[][] tree=new int[depth][nodeCount];
    int[] treeSize=new int[depth];
    Random rnd=new Random();
    for (int i=0;i<depth;i++)
      treeSize[i]=0;
    for (int i=0;i<nodeCount;i++)
    {
      if (i<depth)
      {
        treeSize[i]++;
      }
      else
      {
        int a=0;
        if (depth>1)
          a=rnd.nextInt(depth-1)+1;
        treeSize[a]++;
      }
    }
    for (int i=1;i<depth;i++)
    {
      for (int j=0;j<treeSize[i];j++)
      {
        tree[i][j]=rnd.nextInt(treeSize[i-1]);
      }
      int[] row=new int[treeSize[i]];
      System.arraycopy(tree[i], 0, row, 0, treeSize[i]);
      tree[i]=row;
      Arrays.sort(tree[i]);
    }
    Node[][] treeNodes=new Node[depth][];
    int minX=Layout.MinHorizontalPosition(this);
    int minY=Layout.MinVerticalPosition(this);
    int maxX=Layout.MaxHorizontalPosition(this, d);
    int maxY=Layout.MaxVerticalPosition(this, d);
    int dy=(maxY-minY)/2;
    if (depth>1)
      dy=(maxY-minY)/(depth-1);
    int y=(maxY+minY)/2;
    if (depth>1)
      y=minY;
    nodes=new Node[nodeCount];
    int num=0;
    for (int i=0;i<depth;i++)
    {
      int dx=(maxX-minX)/2;
      if (treeSize[i]>1)
        dx=(maxX-minX)/(treeSize[i]-1);
      int x=(maxX+minX)/2;
      if (treeSize[i]>1)
        x=minX;
      treeNodes[i]=new Node[treeSize[i]];
      for (int j=0;j<treeSize[i];j++)
      {
        treeNodes[i][j]=CreateNode();
        nodes[num]=treeNodes[i][j];
        treeNodes[i][j].SetPosition(new Point(x, y));
        if (i>0)
        {
          AddLink(treeNodes[i][j], treeNodes[i-1][tree[i][j]]);
        }
        x+=dx;
        num++;
      }
      y+=dy;
    }
    ActualiseIdMappingIndeces();
    ShuffleLinks();
    Layout.CenterLayout(this, d);
  }
  
  protected boolean TestBipartite(Node node, int color, int[] result)
  {
    if (node!=null)
    {
      int index=GetNodeIndex(node);
      if (result[index]==0)
      {
        result[index]=color;
        if (color==1) color=2; else color=1;
        for (int i=0;i<node.links.length;i++)
        {
          if (TestBipartite(node.links[i].GetOtherNode(node),color,result)==false) 
            return false;
        }
        return true;
      }
      else 
      {
        return result[index]==color;
      }
    }
    return true;
  }
  
  public boolean TestBipartite()
  {
    int[] set=new int[nodes.length];
    boolean result=true;
    for (int i=0;i<nodes.length;i++)
    {
      set[i]=0;
    }
    for (int i=0;i<nodes.length;i++)
    {
      if (set[i]==0)
      {
        result=TestBipartite(nodes[i],1,set);
        if (!result) return false;
      }
    }
    return true;
  }
  
  public boolean TestComplete()
  {
    return links.length==(nodes.length*(nodes.length-1)/2);
  }
  
  public void PrepareForVisualization(Player player)
  {
    ActualiseIdMappingIndeces();
    for (int i=0;i<nodes.length;i++)
    {
      nodes[i].PrepareForVisualization(player);
    }
    for (int i=0;i<links.length;i++)
    {
      links[i].PrepareForVisualization();
    }
  }
  
  public void ShuffleLinks()
  {
    for (int i=0;i<nodes.length;i++)
    {
      nodes[i].ShuffleLinks();
    }
  }
  
}
