/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package network;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import player.Player;

/**
 *
 * @author Dur
 */
public class NetworkConstraint extends Constraint
{
  public static volatile int numActualMessages=0;
  public static volatile int numSentMessages=0;
  public static HashMap<Integer, Integer> numActualMessagesOfType;
  public static HashMap<Integer, Integer> numSentMessagesOfType;
  public static int[] numSentMessagesByNode;
  public static HashMap<Integer,Integer>[] numSentMessagesOfTypeByNode;
  
  public Integer messageNum=null;
  public Integer messageType=null;
  public Integer currentMessageType=null;
  public int currentMessageOperator=2;
  public int currentMessageCount=0;
  
  public NetworkConstraint()
  {
  }
  
  public static void Initialize(Player player)
  {
    numActualMessages=0;
    numSentMessages=0;
    numActualMessagesOfType=new HashMap<Integer, Integer>();
    numSentMessagesOfType=new HashMap<Integer, Integer>();
    numSentMessagesByNode=new int[player.network.nodes.length];
    for (int i=0;i<player.network.nodes.length;i++)
    {
      numSentMessagesByNode[i]=0;
    }
    numSentMessagesOfTypeByNode=new HashMap[player.network.nodes.length];
    for (int i=0;i<player.network.nodes.length;i++)
    {
      numSentMessagesOfTypeByNode[i]=new HashMap<Integer, Integer>();
    }
  }
  
  @Override
  public void RunBeforeVisualization(Player player)
  {
    super.RunBeforeVisualization(player);
  }
  
  @Override
  public boolean IsSatisfied(LinkMessage mes)
  {
    if (mes!=null)
    {
      if ((messageType!=null) && (!messageType.equals(mes.message.GetType())))
      {
        //if message type is different from message type bound by constraint
        return true;
      }
      
      //find out whether serial number of message corresponds
      if (messageNum!=null)
      {
        Integer sn=0;
        if (messageType==null)
        {
          synchronized (numSentMessagesByNode)
          {
            sn=numSentMessagesByNode[mes.sender.network.GetNodeIndex(mes.sender)];
          }
        }
        else
        {
          synchronized (numSentMessagesOfTypeByNode)
          {
            sn=numSentMessagesOfTypeByNode[mes.sender.network.GetNodeIndex(mes.sender)].get(mes.message.GetType());
          }
        }
        if ((messageNum!=null) && (!messageNum.equals(sn)))
        {
          //if serial number of message differs from constraint serial number
          return true;
        }
      }
       
      int currentMessageCount2=0;
      if (currentMessageType==null)
      {
        currentMessageCount2=numActualMessages;
      }
      else
      {
        synchronized (numActualMessagesOfType)
        {
          Integer num=numActualMessagesOfType.get(currentMessageType);
          if (num!=null)
            currentMessageCount2=num;
        }
      }
      switch (currentMessageOperator)
      {
        case 0: 
          return currentMessageCount2<currentMessageCount;
        case 1: 
          return currentMessageCount2<=currentMessageCount;
        case 2: 
          return currentMessageCount2==currentMessageCount;
        case 3: 
          return currentMessageCount2>=currentMessageCount;
        case 4: 
          return currentMessageCount2>currentMessageCount;
      }
    }
    return true;
  }
  
  public static void SentMessage(LinkMessage mes)
  {
    numActualMessages++;
    numSentMessages++;
    synchronized (numActualMessagesOfType)
    {
      Integer num=numActualMessagesOfType.get(mes.message.GetType());
      if (num==null)
      {
        num=0;
      }
      num=num+1;
      numActualMessagesOfType.put(mes.message.GetType(), num);
    }
    synchronized (numSentMessagesOfType)
    {
      Integer num=numSentMessagesOfType.get(mes.message.GetType());
      if (num==null)
      {
        num=0;
      }
      num=num+1;
      numSentMessagesOfType.put(mes.message.GetType(), num);
    }
    int nodeIndex=mes.sender.network.GetNodeIndex(mes.sender);
    synchronized (numSentMessagesByNode)
    {
      numSentMessagesByNode[nodeIndex]+=1;
    }
    synchronized (numSentMessagesOfTypeByNode)
    {
      Integer num=numSentMessagesOfTypeByNode[nodeIndex].get(mes.message.GetType());
      if (num==null)
      {
        num=0;
      }
      num+=1;
      numSentMessagesOfTypeByNode[nodeIndex].put(mes.message.GetType(), num);
    }
  }
  
  public static void ReceivedMessage(LinkMessage mes)
  {
    numActualMessages--;
    synchronized (numActualMessagesOfType)
    {
      Integer num=numActualMessagesOfType.get(mes.message.GetType());
      if (num==null)
      {
        num=0;
      }
      num=num-1;
      numActualMessagesOfType.put(mes.message.GetType(), num);
    }
  }
  
  public static void DestroyedMessage(LinkMessage mes)
  {
    numActualMessages--;
    synchronized (numActualMessagesOfType)
    {
      Integer num=numActualMessagesOfType.get(mes.message.GetType());
      if (num==null)
      {
        num=0;
      }
      num=num-1;
      numActualMessagesOfType.put(mes.message.GetType(), num);
    }
  }
  
  public static int GetSentMessageCount(int type)
  {
    int res=0;
    synchronized (numSentMessagesOfType)
    {
      Integer num=numSentMessagesOfType.get(type);
      if (num!=null)
        res=num;
    }
    return res;
  }
  
  public static int GetSentMessageCount(int type, int nodeIndex)
  {
    int res=0;
    if ((nodeIndex>=0) && (nodeIndex<numSentMessagesOfTypeByNode.length))
    {
      synchronized (numSentMessagesOfTypeByNode)
      {
        Integer num=numSentMessagesOfTypeByNode[nodeIndex].get(type);
        if (num!=null)
          res=num;
      }
    }
    return res;
  }
  
  public static int GetActualMessageCount(int type)
  {
    int res=0;
    synchronized (numActualMessagesOfType)
    {
      Integer num=numActualMessagesOfType.get(type);
      if (num!=null)
        res=num;
    }
    return res;
  }
  
  public static Integer[] GetSentMessageTypes()
  {
    Integer[] res=new Integer[0];
    LinkedList<Integer> list=new LinkedList<Integer>();
    synchronized (numSentMessagesOfType)
    {
      Iterator<Integer> it=numSentMessagesOfType.keySet().iterator();
      while (it.hasNext())
      {
        list.add(it.next());
      }
      res=list.toArray(new Integer[0]);
      Arrays.sort(res);
    }
    return res;
  }
  
  public static Integer[] GetSentMessageTypesByNode(int nodeIndex)
  {
    Integer[] res=new Integer[0];
    if ((nodeIndex>=0) && (nodeIndex<numSentMessagesOfTypeByNode.length))
    {
      LinkedList<Integer> list=new LinkedList<Integer>();
      synchronized (numSentMessagesOfTypeByNode)
      {
        Iterator<Integer> it=numSentMessagesOfTypeByNode[nodeIndex].keySet().iterator();
        while (it.hasNext())
        {
          list.add(it.next());
        }
        res=list.toArray(new Integer[0]);
        Arrays.sort(res);
      }
    }
    return res;
  }
  
}
