/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package thread;

import algorithm.MessageReceivedListener;
import algorithm.StopException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map.Entry;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;
import network.Constraint;
import network.Link;
import network.LinkMessage;
import network.Message;
import network.Node;
import other.AssociativeArray;
import other.Functions;
import other.Global;
import player.Player;

/**
 *
 * @author Dur
 */
public class LinksThread extends CommunicatingThread
{
  protected AssociativeArray<Node, LinkedList<LinkMessage>> nodeMessBuf=new AssociativeArray<Node, LinkedList<LinkMessage>>();
  public Player player;
  public TreeMap<Long, LinkedList<LinkMessage>> activeMessages=new TreeMap<Long, LinkedList<LinkMessage>>();
  public HashMap<Node,HashMap<Node,LinkedList<LinkMessage>>> constraints=new HashMap<Node, HashMap<Node, LinkedList<LinkMessage>>>();
  
  public LinksThread(Player player)
  {
    this.player=player;
    Constraint.Initialize(player);
  }
  
  private void PutMessageIntoNodeBuffer(Node node, LinkMessage mes)
  {
    try
    {
      mes.message.SetLinkNum(mes.recipient.GetLinkId(mes.link));
      synchronized (nodeMessBuf)
      {
        LinkedList<LinkMessage> list=nodeMessBuf.get(node);
        if (list==null)
        {
          list=new LinkedList<LinkMessage>();
          nodeMessBuf.put(node, list);
        }
        list.add(mes);
      }
      Constraint.ReceivedMessage(mes);
      node.nodeThread.PutObjectLast(Player.MESSAGE_WAKEUP);
    }
    catch (Exception e)
    {
      Global.errorLog.Write("Error: LinksThread->PutMessageIntoNodeBuffer", e);
    }
  }
  
  private int Recheck()
  {
    try
    {
      int num=0;
      boolean iterate=true;
      LinkedList<LinkMessage> newMessages=new LinkedList<LinkMessage>();
      synchronized (activeMessages)
      {
        SortedMap<Long, LinkedList<LinkMessage>> mesList=activeMessages.headMap(player.GetRunningTime());
        Iterator<Entry<Long, LinkedList<LinkMessage>>> it=mesList.entrySet().iterator();
        while (it.hasNext() && (iterate))
        {
          Entry<Long, LinkedList<LinkMessage>> e=it.next();
          if (e.getKey().longValue()>player.GetRunningTime())
            iterate=false;
          if (iterate)
          {
            LinkedList<LinkMessage> list=e.getValue();
            synchronized (list)
            {
              Iterator<LinkMessage> itMes=list.iterator();
              while (itMes.hasNext())
              {
                LinkMessage mes=itMes.next();
                LinkedList<LinkMessage> linkBuf=mes.link.GetMessageBuffer(mes.recipient);
                synchronized (linkBuf)
                {
                  if (linkBuf.getFirst()==mes)
                  {
                    LinkMessage mesSave=linkBuf.remove();
                    LinkMessage newMes=mesSave.GetNextLinkMessage();
                    num++;
                    if (newMes!=null)
                    {
                      //if message should route further
                      newMessages.add(newMes);
                      Constraint.ReceivedMessage(mesSave);
                    }
                    else
                    {
                      PutMessageIntoNodeBuffer(mesSave.recipient, mesSave);
                    }
                    itMes.remove();
                  }
                }
              }
              if (list.size()==0)
              {
                it.remove();
              }
            }
          }
        }
      }
      //save new messages
      if (newMessages.size()>0)
      {
        Iterator<LinkMessage> itMes=newMessages.iterator();
        while (itMes.hasNext())
        {
          LinkMessage newMes=itMes.next();
          SaveLinkMessage(newMes);
        }
        PutObjectLast(Player.MESSAGE_WAKEUP);
      }
      return num;
    }
    catch (Exception e)
    {
      Global.errorLog.Write("Error: LinksThread->Recheck", e);
    }
    return 0;
  }
  
  private void ActivateMessage(LinkMessage mes)
  {
    try
    {
      mes.link.AppendMessage(mes);
      synchronized (activeMessages)
      {
        LinkedList<LinkMessage> list=activeMessages.get(mes.completionMilisec);
        if (list==null)
        {
          list=new LinkedList<LinkMessage>();
          activeMessages.put(mes.completionMilisec, list);
        }
        synchronized (list)
        {
          list.add(mes);
        }
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
      Global.errorLog.Write("Error: LinksThread->ActivateMessage", e);
    }
  }
  
  private void SaveLinkMessage(LinkMessage mes)
  {
    try
    {
      //find out whether message can be send
      boolean sendMessage=Functions.BooleanValDef(mes.link.prop.get("active").toString(), true);
      if (!sendMessage) 
      {
        int activationTimer=Functions.intVal(mes.link.prop.get("activationTimer").toString());
        if ((activationTimer>0) && (activationTimer<=player.GetRunningTime()))
          sendMessage=true;
      }
      
      Constraint.SentMessage(mes);
      if (sendMessage)
      {
        boolean constraintsKept=true;
        synchronized (constraints)
        {
          HashMap<Node,LinkedList<LinkMessage>> map=constraints.get(mes.sender);
          if (map!=null)
          {
            LinkedList<LinkMessage> list=map.get(mes.recipient);
            if (list!=null)
            {
              if (list.size()>0)
                constraintsKept=false;
            }
          }
        }
        if (constraintsKept)
        {
          for (int i=0;i<mes.link.constraints.length;i++)
          {
            constraintsKept&=mes.link.constraints[i].IsSatisfied(mes);
            if (!constraintsKept)
              break;
          }
        }
        if (constraintsKept)
        {
          ActivateMessage(mes);
        }
        else
        {
          //message should be placed in waiting queue
          PutMessageInQueue(mes);
        }
        player.linksThread.PutObjectLast(Player.MESSAGE_WAKEUP);
      }
      else
      {
        Constraint.DestroyedMessage(mes);
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
      Global.errorLog.Write("Error: LinksThread->SaveLinkMessage", e);
    }
  }
  
  private void PutMessageInQueue(LinkMessage mes)
  {
    try
    {
      synchronized (constraints)
      {
        HashMap<Node,LinkedList<LinkMessage>> map=constraints.get(mes.sender);
        if (map==null)
        {
          map=new HashMap<Node, LinkedList<LinkMessage>>();
          constraints.put(mes.sender, map);
        }
        LinkedList<LinkMessage> list=map.get(mes.recipient);
        if (list==null)
        {
          list=new LinkedList<LinkMessage>();
          map.put(mes.recipient, list);
        }
        list.add(mes);
      }
    }
    catch (Exception e)
    {
      Global.errorLog.Write("Error: LinksThread->PutMessageInQueue", e);
    }
  }
  
  private void ActivateWaitingMessages()
  {
    try
    {
      synchronized (constraints)
      {
        Iterator<HashMap<Node,LinkedList<LinkMessage>>> it=constraints.values().iterator();
        while (it.hasNext())
        {
          HashMap<Node,LinkedList<LinkMessage>> map=it.next();
          Iterator<LinkedList<LinkMessage>> it2=map.values().iterator();
          while (it2.hasNext())
          {
            LinkedList<LinkMessage> list=it2.next();
            Iterator<LinkMessage> itMes=list.iterator();
            boolean canSend=true;
            while (canSend && itMes.hasNext())
            {
              LinkMessage mes=itMes.next();
              boolean isSatisfied=true;
              for (int i=0;i<mes.link.constraints.length;i++)
              {
                isSatisfied&=mes.link.constraints[i].IsSatisfied(mes);
                if (!isSatisfied)
                  break;
              }
              if (isSatisfied)
              {
                mes.creationMilisec=player.GetRunningTime();
                mes.completionMilisec=mes.GetCompletionMilisec(mes.link, mes.creationMilisec);
                ActivateMessage(mes);
                itMes.remove();
              }
              else
                canSend=false;
            }
            if (list.size()==0)
              it2.remove();
          }
          if (map.size()==0)
            it.remove();
        }
      }
    }
    catch (Exception e)
    {
      Global.errorLog.Write("Error: LinksThread->ActivateWaitingMessages", e);
    }
  }
  
  public void SendMessage(Node sender, Link link, Message mes)
  {
    try
    {
      if ((sender!=null) && (link!=null) && (mes!=null))
      {
        LinkMessage lMes=new LinkMessage(player, sender, link, mes);
        SaveLinkMessage(lMes);
      }
    }
    catch (Exception e)
    {
      Global.errorLog.Write("Error: LinksThread->SendMessage", e);
    }
  }
  
  public void SendMessage(Node sender, Node recipient, Message mes)
  {
    try
    {
      if ((sender!=null) && (recipient!=null) && (mes!=null))
      {
        LinkMessage lMes=new LinkMessage(player, sender, recipient, mes);
        SaveLinkMessage(lMes);
      }
    }
    catch (Exception e)
    {
      Global.errorLog.Write("Error: LinksThread->SendMessage", e);
    }
  }
  
  public boolean IsMessageInBuffer(Node node)
  {
    try
    {
      LinkedList<LinkMessage> list;
      synchronized (nodeMessBuf)
      {
        list=nodeMessBuf.get(node);
      }
      if (list!=null)
      {
        boolean res=false;
        synchronized (list)
        {
          res=list.size()>0;
        }
        return res;
      }
    }
    catch (Exception e)
    {
      Global.errorLog.Write("Error: LinksThread->IsMessageInBuffer", e);
    }
    return false;
  }
  
  public LinkMessage ReceiveMessage(Node node, HashMap<Integer,MessageReceivedListener> listeners)
  {
    try
    {
      LinkedList<LinkMessage> list;
      LinkMessage res=null;
      LinkedList<LinkMessage> receiveMessages=new LinkedList<LinkMessage>();
      synchronized (nodeMessBuf)
      {
        list=nodeMessBuf.get(node);
        if (list!=null)
        {
          synchronized (list)
          {
            receiveMessages=ReceiveMessagesViaListeners(node, listeners);
            if (list.size()>0)
              res=list.remove();
          }
        }
      }
      Iterator<LinkMessage> it=receiveMessages.iterator();
      while (it.hasNext())
      {
        LinkMessage mes=it.next();
        MessageReceivedListener listener=listeners.get(mes.message.GetType());
        if (listener!=null)
        {
          listener.OnMessageReceive(mes.message);
        }
      }
      return res;
    }
    catch (StopException e)
    {
      throw e;
    }
    catch (Exception e)
    {
      Global.errorLog.Write("Error: LinksThread->ReceiveMessage", e);
    }
    return null;
  }
  
  public LinkMessage ReceiveMessage(Node node, int linkNum, HashMap<Integer,MessageReceivedListener> listeners)
  {
    try
    {
      LinkedList<LinkMessage> list;
      LinkedList<LinkMessage> receiveMessages=new LinkedList<LinkMessage>();
      LinkMessage res=null;
      boolean lookFor=true;
      synchronized (nodeMessBuf)
      {
        list=nodeMessBuf.get(node);
        if (list!=null)
        {
          synchronized (list)
          {
            receiveMessages=ReceiveMessagesViaListeners(node, listeners);
            Iterator<LinkMessage> it=list.iterator();
            while (it.hasNext() && lookFor)
            {
              res=it.next();
              if (res.message.GetLinkNum()==linkNum)
              {
                lookFor=false;
                it.remove();
              }
            }
          }
        }
      }
      Iterator<LinkMessage> it=receiveMessages.iterator();
      while (it.hasNext())
      {
        LinkMessage mes=it.next();
        MessageReceivedListener listener=listeners.get(mes.message.GetType());
        if (listener!=null)
        {
          listener.OnMessageReceive(mes.message);
        }
      }
      if (!lookFor)
        return res;
    }
    catch (StopException e)
    {
      throw e;
    }
    catch (Exception e)
    {
      Global.errorLog.Write("Error: LinksThread->ReceiveMessage", e);
    }
    return null;
  }
  
  public LinkMessage ReceiveMessage(Node node, Set<Integer> linkSet, HashMap<Integer,MessageReceivedListener> listeners)
  {
    try
    {
      LinkedList<LinkMessage> list;
      LinkedList<LinkMessage> receiveMessages=new LinkedList<LinkMessage>();
      LinkMessage res=null;
      boolean lookFor=true;
      synchronized (nodeMessBuf)
      {
        list=nodeMessBuf.get(node);
        if (list!=null)
        {
          synchronized (list)
          {
            receiveMessages=ReceiveMessagesViaListeners(node, listeners);
            Iterator<LinkMessage> it=list.iterator();
            while (it.hasNext() && lookFor)
            {
              res=it.next();
              if (linkSet.contains(res.message.GetLinkNum()))
              {
                lookFor=false;
                it.remove();
              }
            }
          }
        }
      }
      Iterator<LinkMessage> it=receiveMessages.iterator();
      while (it.hasNext())
      {
        LinkMessage mes=it.next();
        MessageReceivedListener listener=listeners.get(mes.message.GetType());
        if (listener!=null)
        {
          listener.OnMessageReceive(mes.message);
        }
      }
      if (!lookFor)
        return res;
    }
    catch (StopException e)
    {
      throw e;
    }
    catch (Exception e)
    {
      Global.errorLog.Write("Error: LinksThread->ReceiveMessage", e);
    }
    return null;
  }
  
  //this function assumes that nodeMessBuf is synchronized
  public LinkedList<LinkMessage> ReceiveMessagesViaListeners(Node node, HashMap<Integer,MessageReceivedListener> listeners)
  {
    try
    {
      boolean[] canUseLink=new boolean[node.links.length];
      for (int i=0;i<canUseLink.length;i++)
        canUseLink[i]=true;
      
      LinkedList<LinkMessage> list;
      LinkedList<LinkMessage> receiveMessages=new LinkedList<LinkMessage>();
      LinkMessage res=null;
      list=nodeMessBuf.get(node);
      if (list!=null)
      {
        Iterator<LinkMessage> it=list.iterator();
        while (it.hasNext())
        {
          res=it.next();
          if (canUseLink[res.message.GetLinkNum()] && (listeners.get(res.message.GetType()))!=null)
          {
            receiveMessages.add(res);
            it.remove();
          }
          else
          {
            canUseLink[res.message.GetLinkNum()]=false;
          }
        }
      }
      return receiveMessages;
    }
    catch (Exception e)
    {
      Global.errorLog.Write("Error: LinksThread->ReceiveMessagesViaListeners", e);
    }
    return new LinkedList<LinkMessage>();
  }
  
  @Override
  public void run()
  {
    try
    {
      Object mes;
      do
      {
        //check whether some constraints are satisfied and activate them
        ActivateWaitingMessages();
        
        //check which messages has passed through canUseLink
        if (Recheck()>0)
          PutObjectLast(Player.MESSAGE_WAKEUP);
        
        //wait for a message signal or wait until a first message is ready to be sent
        Long firstTime=null;
        synchronized(activeMessages)
        {
          if (activeMessages.size()>0)
            firstTime=activeMessages.firstKey();
        }
        if (firstTime==null)
        {
          mes=ReceiveObject();
        }
        else
        {
          int milisec=Math.abs((int)(firstTime.longValue()-(player.GetRunningTime())))+5;
          mes=ReceiveObject(milisec);
        }
        
        //check whether player is not paused
        if ((mes!=null) && mes.equals(Player.MESSAGE_PAUSE))
        {
          while ((mes==null) || ((!mes.equals(Player.MESSAGE_START)) && (!mes.equals(Player.MESSAGE_STOP))))
          {
            mes=ReceiveObject();
          }
        }
      }
      while ((mes==null) || (!mes.equals(Player.MESSAGE_STOP)));
    }
    catch (Exception e)
    {
      Global.errorLog.Write("Error: LinksThread->run", e);
    }
  }
  
  
}
